import collections

from eventlet import api, greenlib

import greenlet

__all__ = ['channel']

class channel(object):
    balance = 0

    def _tasklet_loop(self):
        deque = self.deque = collections.deque()
        hub = api.get_hub()
        switch = greenlib.switch
        direction, caller, args = switch()
        try:
            while True:
                if direction == -1:
                    # waiting to receive
                    if self.balance > 0:
                        sender, args = deque.popleft()
                        hub.schedule_call(0, switch, sender)
                        hub.schedule_call(0, switch, caller, *args)
                    else:
                        deque.append(caller)
                else:
                    # waiting to send
                    if self.balance < 0:
                        receiver = deque.popleft()
                        hub.schedule_call(0, switch, receiver, *args)
                        hub.schedule_call(0, switch, caller)
                    else:
                        deque.append((caller, args))
                self.balance += direction
                direction, caller, args = hub.switch()
        finally:
            deque.clear()
            del self.deque
            self.balance = 0

    def _send_tasklet(self, *args):
        try:
            t = self._tasklet
        except AttributeError:
            t = self._tasklet = greenlib.tracked_greenlet()
            greenlib.switch(t, (self._tasklet_loop,))
        if args:
            return greenlib.switch(t, (1, greenlet.getcurrent(), args))
        else:
            return greenlib.switch(t, (-1, greenlet.getcurrent(), args))
        
    def receive(self):
        return self._send_tasklet()

    def send(self, value):
        return self._send_tasklet(value)

    def send_exception(self, exc):
        return self._send_tasklet(None, exc)



"""runloop.py

Defines the core eventlet runloop. The runloop keeps track of scheduled
events and observers which watch for specific portions of the runloop to
be executed. 
"""

import time
import bisect
import sys
import traceback

from eventlet.timer import Timer


class RunLoop(object):
    SYSTEM_EXCEPTIONS = (KeyboardInterrupt, SystemExit)
    
    def __init__(self, wait=None, clock=None):
        if clock is None:
            clock = self.default_clock()
        self.clock = clock
        if wait is None:
            wait = self.default_wait
        self.wait = wait
        self.stopping = False
        self.running = False
        self.timers = []
        self.next_timers = []
        self.observers = {}
        self.observer_modes = {
            'entry': [],
            'before_timers': [],
            'before_waiting': [],
            'after_waiting': [],
            'exit': [],
        }

    def default_wait(self, time):
        return None
        
    def default_clock(self):
        return time.time

    def default_sleep(self):
        return 60.0

    def sleep_until(self):
        t = self.timers
        if not t:
            return None
        return t[0][0]
        
    def run(self):
        """Run the runloop until abort is called.
        """
        if self.running:
            raise RuntimeError("Already running!")
        try:
            self.running = True
            self.stopping = False
            self.fire_observers('entry')
            while not self.stopping:
                self.prepare_timers()
                self.fire_observers('before_timers')
                self.fire_timers(self.clock())
                self.prepare_timers()
                wakeup_when = self.sleep_until()
                if wakeup_when is None:
                    sleep_time = self.default_sleep()
                else:
                    sleep_time = wakeup_when - self.clock()
                if sleep_time > 0:
                    self.fire_observers('before_waiting')
                    self.wait(sleep_time)
                    self.fire_observers('after_waiting')
                else:
                    self.wait(0)
            else:
                del self.timers[:]
                del self.next_timers[:]
            self.fire_observers('exit')
        finally:
            self.running = False
            self.stopping = False

    def abort(self):
        """Stop the runloop. If run is executing, it will exit after completing
        the next runloop iteration.
        """
        if self.running:
            self.stopping = True

    def add_observer(self, observer, *modes):
        """Add an event observer to this runloop with the given modes.
        Valid modes are:
            entry: The runloop is being entered.
            before_timers: Before the expired timers for this iteration are executed.
            before_waiting: Before waiting for the calculated wait_time
                where nothing will happen.
            after_waiting: After waiting, immediately before starting the top of the
                runloop again.
            exit: The runloop is exiting.

        If no mode is passed or mode is all, the observer will be fired for every
        event type.
        """
        if not modes or modes == ('all',):
            modes = tuple(self.observer_modes)
        self.observers[observer] = modes
        for mode in modes:
            self.observer_modes[mode].append(observer)

    def remove_observer(self, observer):
        """Remove a previously registered observer from all event types.
        """
        for mode in self.observers.pop(observer, ()):
            self.observer_modes[mode].remove(observer)
            
    def squelch_observer_exception(self, observer, exc_info):
        traceback.print_exception(*exc_info)
        print >>sys.stderr, "Removing observer: %r" % (observer,)
        self.remove_observer(observer)
        
    def fire_observers(self, activity):
        for observer in self.observer_modes[activity]:
            try:
                observer(self, activity)
            except self.SYSTEM_EXCEPTIONS:
                raise
            except:
                self.squelch_observer_exception(observer, sys.exc_info())

    def squelch_timer_exception(self, timer, exc_info):
        traceback.print_exception(*exc_info)
        print >>sys.stderr, "Timer raised: %r" % (timer,)

    def _add_absolute_timer(self, when, info):
        # the 0 placeholder makes it easy to bisect_right using (now, 1)
        self.next_timers.append((when, 0, info))

    def add_timer(self, timer):
        scheduled_time = self.clock() + timer.seconds
        self._add_absolute_timer(scheduled_time, timer)
        return scheduled_time
    
    def prepare_timers(self):
        ins = bisect.insort_right
        t = self.timers
        for item in self.next_timers:
            ins(t, item)
        del self.next_timers[:]

    def schedule_call(self, seconds, cb, *args, **kw):
        """Schedule a callable to be called after 'seconds' seconds have
        elapsed.
            seconds: The number of seconds to wait.
            cb: The callable to call after the given time.
            *args: Arguments to pass to the callable when called.
            **kw: Keyword arguments to pass to the callable when called.
        """
        t = Timer(seconds, cb, *args, **kw)
        self.add_timer(t)
        return t

    def fire_timers(self, when):
        t = self.timers
        last = bisect.bisect_right(t, (when, 1))
        i = 0
        for i in xrange(last):
            timer = t[i][2]
            try:
                timer()
            except self.SYSTEM_EXCEPTIONS:
                raise
            except:
                self.squelch_timer_exception(timer, sys.exc_info())
        del t[:last]



import sys
from code import InteractiveConsole
from eventlet import greenlib

try:
    sys.ps1
except AttributeError:
    sys.ps1 = '>>> '
try:
    sys.ps2
except AttributeError:
    sys.ps2 = '... '

class SocketConsole(greenlib.GreenletContext):
    def __init__(self, desc):
        # mangle the socket
        self.desc = desc
        readline = desc.readline
        self.old = {}
        self.fixups = {
            'softspace': 0,
            'isatty': lambda: True,
            'flush': lambda: None,
            'readline': lambda *a: readline(*a).replace('\r\n', '\n'),
        }
        for key, value in self.fixups.iteritems():
            if hasattr(desc, key):
                self.old[key] = getattr(desc, key)
            setattr(desc, key, value)

    def finalize(self):
        # restore the state of the socket
        for key in self.fixups:
            try:
                value = self.old[key]
            except KeyError:
                delattr(self.desc, key)
            else:
                setattr(self.desc, key, value)
        self.fixups.clear()
        self.old.clear()
        self.desc = None

    def swap_in(self):
        self.saved = sys.stdin, sys.stderr, sys.stdout
        sys.stdin = sys.stdout = sys.stderr = self.desc

    def swap_out(self):
        sys.stdin, sys.stderr, sys.stdout = self.saved

def backdoor((conn, addr), locals=None):
    host, port = addr
    print "backdoor to %s:%s" % (host, port)
    ctx = SocketConsole(conn)
    ctx.register()
    try:
        console = InteractiveConsole(locals)
        console.interact()
    finally:
        ctx.unregister()





import os.path
import popen2
import time

from eventlet import api

from mulib import mu
from mulib import eventrouter

import simplejson

from indra import llsd


from os.path import dirname, join
TEST_SCRIPT_PATH = join(
    join(
        dirname(dirname(dirname(dirname(__file__)))),
    'test'), 'test.py')


class TestConsole(eventrouter.Router):
    transforms = os.path.join(os.path.dirname(__file__), 'testconsole.js')
    stylesheet = os.path.join(os.path.dirname(__file__), 'testconsole.css')

    def __init__(self, my_host):
        self.my_host = my_host
        self.child_ = self
        self.replay = []
        eventrouter.Router.__init__(self)

    def event_connect(self, target):
        for event in self.replay:
            target.msg(*event)
        return "testconsole running on %s" % (self.my_host, )

    def clear_replay(self):
        self.replay = []

    def log_and_broadcast(self, action, data):
        self.replay.append((action, data))
        self.broadcast(action, data)

    def replay_target(self, target):
        for action, data in self.replay:
            target.msg(action, data)

    def handle_post(self, req):
        action = req.get_query('post', 'default')
        data = req.read_body()
        #print action, data,
        ct = req.get_header('content-type')
        if ct == 'application/json':
            data = simplejson.loads(data)
        else:
            data = llsd.parse(data)
        meth = getattr(self, 'post_' + action, None)
        if meth is None:
            meth = lambda *args, **kw: self.log_and_broadcast(action, data)
        if isinstance(data, list):
            meth(*data)
        elif isinstance(data, dict):
            meth(**data)
        elif data is None:
            meth()
        else:
            meth(data)
        if ct == 'application/json':
            req.write('null')
        else:
            req.write(mu.xml('<llsd><undef /></llsd>'))

    def post_log_start(self, id, start_time):
        self.clear_replay()
        self.log_and_broadcast('log_start', [id, start_time])

    def _run_cmd(self, switch, value):
        child = popen2.Popen3([
            '/usr/bin/python2.3',
            TEST_SCRIPT_PATH,
            switch, value,
            '-c', 'http://localhost:12040/test/'
            ])
        status = -1
        while status == -1:
            status = child.poll()
            api.sleep()

    def post_run(self, test_name):
        self._run_cmd('-t', test_name)

    def post_run_suite(self, mode):
        self._run_cmd('-m', mode)




"""wsgi.py
"""

import sys
import time
import urllib
import socket
import cStringIO
import SocketServer
import BaseHTTPServer

from eventlet import api
from eventlet.httpdate import format_date_time

class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):

    def log_message(self, format, *args):
        self.server.log_message("%s - - [%s] %s" % (
            self.address_string(),
            self.log_date_time_string(),
            format % args))

    def handle_one_request(self):
        self.raw_requestline = self.rfile.readline()
    
        if not self.raw_requestline:
            self.close_connection = 1
            return

        if not self.parse_request():
            return

        self.environ = self.get_environ()
        try:
            self.handle_one_response()
        except socket.error, e:
            # Broken pipe, connection reset by peer
            if e[0] in (32, 54):
                pass
            else:
                raise

    def handle_one_response(self):
        headers_set = []
        headers_sent = []
        # set of lowercase header names that were sent
        header_dict = {}

        wfile = self.wfile
        num_blocks = None
        
        def write(data, _write=wfile.write):
            if not headers_set:
                raise AssertionError("write() before start_response()")
            elif not headers_sent:
                status, response_headers = headers_set
                headers_sent.append(1)
                for k, v in response_headers:
                    header_dict[k.lower()] = k
                _write('HTTP/1.0 %s\r\n' % status)
                # send Date header?
                if 'date' not in header_dict:
                    _write('Date: %s\r\n' % (format_date_time(time.time()),))
                if 'content-length' not in header_dict and num_blocks == 1:
                    _write('Content-Length: %s\r\n' % (len(data),))
                for header in response_headers:
                    _write('%s: %s\r\n' % header)
                _write('\r\n')
            _write(data)
                
        def start_request(status, response_headers, exc_info=None):
            if exc_info:
                try:
                    if headers_sent:
                        # Re-raise original exception if headers sent
                        raise exc_info[0], exc_info[1], exc_info[2]
                finally:
                    # Avoid dangling circular ref
                    exc_info = None
            elif headers_set:
                raise AssertionError("Headers already set!")

            headers_set[:] = [status, response_headers]
            return write

        result = self.server.app(self.environ, start_request)
        try:
            num_blocks = len(result)
        except (TypeError, AttributeError, NotImplementedError):
            pass
            
        try:
            for data in result:
                if data:
                    write(data)
            if not headers_sent:
                write('')
        finally:
            if hasattr(result, 'close'):
                result.close()

    def get_environ(self):
        env = self.server.get_environ()
        env['REQUEST_METHOD'] = self.command
        env['SCRIPT_NAME'] = ''

        if '?' in self.path:
            path, query = self.path.split('?', 1)
        else:
            path, query = self.path, ''
        env['PATH_INFO'] = urllib.unquote(path)
        env['QUERY_STRING'] = query

        if self.headers.typeheader is None:
            env['CONTENT_TYPE'] = self.headers.type
        else:
            env['CONTENT_TYPE'] = self.headers.typeheader

        length = self.headers.getheader('content-length')
        if length:
            env['CONTENT_LENGTH'] = length
        env['SERVER_PROTOCOL'] = 'HTTP/1.0'

        host, port = self.request.getsockname()
        env['SERVER_NAME'] = host
        env['SERVER_PORT'] = str(port)
        env['REMOTE_ADDR'] = self.client_address[0]
        env['GATEWAY_INTERFACE'] = 'CGI/1.1'

        for h in self.headers.headers:
            k, v = h.split(':', 1)
            k = k.replace('-', '_').upper()
            v = v.strip()
            if k in env:
                continue
            envk = 'HTTP_' + k
            if envk in env:
                env[envk] += ',' + v
            else:
                env[envk] = v

        return env

    def finish(self):
        # Override SocketServer.StreamRequestHandler.finish because
        # we only need to call close on the socket, not the makefile'd things
        
        self.request.close()


class Server(BaseHTTPServer.HTTPServer):
    def __init__(self, socket, address, app, log, environ=None):
        self.socket = socket
        self.address = address
        if log:
            self.log = log
            log.write = log.info
        else:
            self.log = sys.stderr
        self.app = app
        self.environ = environ

    def get_environ(self):
        socket = self.socket
        d = {
            'wsgi.input': socket,
            'wsgi.errors': sys.stderr,
            'wsgi.version': (1, 0),
            'wsgi.multithread': True,
            'wsgi.multiprocess': False,
            'wsgi.run_once': False,
            'wsgi.url_scheme': 'http',
        }
        if self.environ is not None:
            d.update(self.environ)
        return d

    def process_request(self, (socket, address)):
        proto = HttpProtocol(socket, address, self)

    def log_message(self, message):
        self.log.write(message + '\n')


def server(socket, site, log=None, environ=None):
    serv = Server(socket, socket.getsockname(), site, log, environ=None)
    try:
        print "wsgi starting up on", socket.getsockname()
        while True:
            try:
                api.spawn(serv.process_request, socket.accept())
            except KeyboardInterrupt:
                api.get_hub().remove_descriptor(socket.fileno())
                print "wsgi exiting"
                break
    finally:
        socket.close()



from eventlet.api import trampoline, get_hub
from eventlet import util

BUFFER_SIZE = 4096

class wrapped_fd(object):
    newlines = '\r\n'
    mode = 'wb+'
    
    def __init__(self, fd):
        self._closed = False
        self.fd = fd
        self._fileno = fd.fileno()
        self.recvbuffer = ''
        self.recvcount = 0
        self.sendcount = 0        

    def getpeername(self, *args, **kw):
        fn = self.getpeername = self.fd.getpeername
        return fn(*args, **kw)

    def getsockname(self, *args, **kw):
        fn = self.getsockname = self.fd.getsockname
        return fn(*args, **kw)

    def listen(self, *args, **kw):
        fn = self.listen = self.fd.listen
        return fn(*args, **kw)

    def bind(self, *args, **kw):
        fn = self.bind = self.fd.bind
        return fn(*args, **kw)

    def getsockopt(self, *args, **kw):
        fn = self.getsockopt = self.fd.getsockopt
        return fn(*args, **kw)

    def setsockopt(self, *args, **kw):
        fn = self.setsockopt = self.fd.setsockopt
        return fn(*args, **kw)

    def connect_ex(self, *args, **kw):
        fn = self.connect_ex = self.fd.connect_ex
        return fn(*args, **kw)

    def fileno(self, *args, **kw):
        fn = self.fileno = self.fd.fileno
        return fn(*args, **kw)

    def setblocking(self, *args, **kw):
        fn = self.setblocking = self.fd.setblocking
        return fn(*args, **kw)

    def close(self, *args, **kw):
        if self._closed:
            return
        self._closed = True
        fn = self.close = self.fd.close
        try:
            res = fn(*args, **kw)
        finally:
            # This will raise socket.error(32, 'Broken pipe') if there's
            # a caller waiting on trampoline (e.g. server on .accept())
            get_hub().exc_descriptor(self._fileno)
        return res

    def accept(self):
        fd = self.fd
        while True:
            res = util.socket_accept(fd)
            if res is not None:
                client, addr = res
                util.set_nonblocking(client)
                return type(self)(client), addr
            trampoline(fd, read=True, write=True)

    def connect(self, address):
        fd = self.fd
        connect = util.socket_connect
        while not connect(fd, address):
            trampoline(fd, read=True, write=True)
    
    def recv(self, buflen):
        buf = self.recvbuffer
        if buf:
            chunk, self.recvbuffer = buf[:buflen], buf[buflen:]
            return chunk
        fd = self.fd
        recv = util.socket_recv
        bytes = recv(fd, buflen)
        while bytes is None:
            trampoline(fd, read=True)
            bytes = recv(fd, buflen)
        self.recvcount += len(bytes)
        return bytes

    def send(self, data):
        count = util.socket_send(self.fd, data)
        if not count:
            return 0
        self.sendcount += count
        return count

    def sendall(self, data):
        fd = self.fd
        tail = self.send(data)
        while tail < len(data):
            trampoline(self.fd, write=True)
            tail += self.send(data[tail:])

    def write(self, data):
        return self.sendall(data)
    
    def readuntil(self, terminator, size=None):
        buf, self.recvbuffer = self.recvbuffer, ''
        checked = 0
        if size is None:
            while True:
                found = buf.find(terminator, checked)
                if found != -1:
                    found += len(terminator)
                    chunk, self.recvbuffer = buf[:found], buf[found:]
                    return chunk
                checked = len(buf)
                d = self.recv(BUFFER_SIZE)
                if not d:
                    break
                buf += d
            return buf
        while len(buf) < size:
            found = buf.find(terminator, checked)
            if found != -1:
                found += len(terminator)
                chunk, self.recvbuffer = buf[:found], buf[found:]
                return chunk
            checked = len(buf)
            d = self.recv(BUFFER_SIZE)
            if not d:
                break
            buf += d
        chunk, self.recvbuffer = buf[:size], buf[size:]
        
    def readline(self, size=None):
        return self.readuntil(self.newlines, size=size)

    def __iter__(self):
        return self.xreadlines()

    def readlines(self, size=None):
        return list(self.xreadlines(size=size))

    def xreadlines(self, size=None):
        if size is None:
            while True:
                line = self.readline()
                if not line:
                    break
                yield line
        else:
            while size > 0:
                line = self.readline(size)
                if not line:
                    break
                yield line
                size -= len(line)

    def writelines(self, lines):
        for line in lines:
            self.write(line)
        
    def read(self, size=None):
        buf, self.recvbuffer = self.recvbuffer, ''
        lst = [buf]
        if size is None:
            while True:
                d = self.recv(BUFFER_SIZE)
                if not d:
                    break
                lst.append(d)
        else:
            buflen = len(buf)
            while buflen < size:
                d = self.recv(BUFFER_SIZE)
                if not d:
                    break
                buflen += len(d)
                lst.append(d)
            else:
                d = lst[-1]
                overbite = buflen - size
                if overbite:
                    lst[-1], self.recvbuffer = d[:-overbite], d[-overbite:]
                else:
                    lst[-1], self.recvbuffer = d, ''
        return ''.join(lst)

    def makefile(self, *args, **kw):
        return type(self)(self.fd)



"""test_timer
"""

import unittest

from eventlet import api
from eventlet import timer
from eventlet import runloop


class TestTimer(unittest.TestCase):
    def test_copy(self):
        t = timer.Timer(0, lambda: None)
        t2 = t.copy()
        assert t.seconds == t2.seconds
        assert t.tpl == t2.tpl
        assert t.called == t2.called

    def test_cancel(self):
        r = runloop.RunLoop()
        called = []
        t = timer.Timer(0, lambda: called.append(True))
        t.cancel()
        r.add_timer(t)
        r.add_observer(lambda r, activity: r.abort(), 'after_waiting')
        r.run()
        assert not called
        assert not r.running

    def test_schedule(self):
        hub = api.get_hub()
        r = hub.runloop
        called = []
        t = timer.Timer(0, lambda: (called.append(True), hub.runloop.abort()))
        t.schedule()
        r.default_sleep = lambda: 0.0
        r.run()
        assert called
        assert not r.running

if __name__ == '__main__':
    unittest.main()



import sys
import select
import errno
import traceback
import time
from bisect import insort, bisect_left

from eventlet import greenlib
from eventlet.runloop import RunLoop, Timer

import greenlet

class Hub(object):
    def __init__(self):
        self.runloop = RunLoop(self.wait)
        self.readers = {}
        self.writers = {}
        self.excs = {}
        self.descriptors = {}
        self.descriptor_queue = {}
        self.greenlet = None

    def stop(self):
        self.process_queue()
        self.runloop.abort()
        if self.greenlet is not greenlet.getcurrent():
            self.switch()

    def schedule_call(self, *args, **kw):
        return self.runloop.schedule_call(*args, **kw)

    def switch(self):
        if not self.greenlet:
            self.greenlet = greenlib.tracked_greenlet()
            args = ((self.runloop.run,),)
        else:
            args = ()
        try:
            greenlet.getcurrent().parent = self.greenlet
        except ValueError:
            pass
        return greenlib.switch(self.greenlet, *args)

    def add_descriptor(self, fileno, read=None, write=None, exc=None):
        self.descriptor_queue[fileno] = read, write, exc

    def remove_descriptor(self, fileno):
        self.descriptor_queue[fileno] = None, None, None
        
    def exc_descriptor(self, fileno):
        # We must handle two cases here, the descriptor
        # may be changing or removing its exc handler
        # in the queue, or it may be waiting on the queue.
        exc = None
        try:
            exc = self.descriptor_queue[fileno][2]
        except KeyError:
            try:
                exc = self.excs[fileno]
            except KeyError:
                pass
        if exc is not None:
            try:
                exc()
            except self.runloop.SYSTEM_EXCEPTIONS:
                self.squelch_exception(fileno, sys.exc_info())

    def squelch_exception(self, fileno, exc_info):
        traceback.print_exception(*exc_info)
        print >>sys.stderr, "Removing descriptor: %r" % (fileno,)
        try:
            self.remove_descriptor(fileno)
        except Exception, e:
            print >>sys.stderr, "Exception while removing descriptor! %r" % (e,)

    def process_queue(self):
        readers = self.readers
        writers = self.writers
        excs = self.excs
        descriptors = self.descriptors
        for fileno, rwe in self.descriptor_queue.iteritems():
            read, write, exc = rwe
            if read is None and write is None and exc is None:
                try:
                    del descriptors[fileno]
                except KeyError:
                    continue
                try:
                    del readers[fileno]
                except KeyError:
                    pass
                try:
                    del writers[fileno]
                except KeyError:
                    pass
                try:
                    del excs[fileno]
                except KeyError:
                    pass
            else:
                if read is not None:
                    readers[fileno] = read
                else:
                    try:
                        del readers[fileno]
                    except KeyError:
                        pass
                if write is not None:
                    writers[fileno] = write
                else:
                    try:
                        del writers[fileno]
                    except KeyError:
                        pass
                if exc is not None:
                    excs[fileno] = exc
                else:
                    try:
                        del excs[fileno]
                    except KeyError:
                        pass
                descriptors[fileno] = rwe
        self.descriptor_queue.clear()
    
    def wait(self, seconds=None):
        self.process_queue()
        if not self.descriptors:
            if seconds:
                time.sleep(seconds)
            return
        readers = self.readers
        writers = self.writers
        excs = self.excs
        try:
            r, w, ig = select.select(readers, writers, [], seconds)
        except select.error, e:
            if e.args[0] == errno.EINTR:
                return
            raise
        SYSTEM_EXCEPTIONS = self.runloop.SYSTEM_EXCEPTIONS
        for observed, events in ((readers, r), (writers, w)):
            for fileno in events:
                try:
                    observed[fileno](fileno)
                except SYSTEM_EXCEPTIONS:
                    raise
                except:
                    self.squelch_exception(fileno, sys.exc_info())



"""httpd.py
"""

import sys
import time
import urllib
import socket
import traceback
import cStringIO
import SocketServer
import BaseHTTPServer

from eventlet import api


USE_ACCESS_LOG = True


class Request(object):
    _responsecode = 200
    _reason_phrase = None
    _request_started = False
    _chunked = False
    def __init__(self, protocol, method, path, headers):
        self.request_start_time = time.time()
        self.site = protocol.server.site
        self.protocol = protocol
        self._method = method
        if '?' in path:
            self._path, self._query = path.split('?', 1)
            self._query = self._query.replace('&amp;', '&')
        else:
            self._path, self._query = path, None
        self._incoming_headers = headers
        self._outgoing_headers = dict()

    def response(self, code, reason_phrase=None):
        """Change the response code. This will not be sent until some
        data is written; last call to this method wins. Default is
        200 if this is not called.
        """
        self._responsecode = code
        self._reason_phrase = reason_phrase
        self.protocol.set_response_code(self, code, reason_phrase)

    def is_okay(self):
        return 200 <= self._responsecode <= 299

    def begin_response(self, length="-"):
        """Begin the response, and return the initial response text
        """
        self._request_started = True
        request_time = time.time() - self.request_start_time

        code = self._responsecode
        proto = self.protocol

        if USE_ACCESS_LOG:
            proto.server.write_access_log_line(
                proto.client_address[0],
                time.strftime("%d/%b/%Y %H:%M:%S"),
                proto.requestline,
                code,
                length,
                request_time)

        if self._reason_phrase is not None:
            message = self._reason_phrase.split("\n")[0]
        elif code in proto.responses:
            message = proto.responses[code][0]
        else:
            message = ''
        if proto.request_version == 'HTTP/0.9':
            return []

        response_lines = proto.generate_status_line()

        if not self._outgoing_headers.has_key('connection'):
            con = self.get_header('connection')
            if con is None:
                con = 'close'
            self.set_header('connection', con)

        for key, value in self._outgoing_headers.items():
            key = '-'.join([x.capitalize() for x in key.split('-')])
            response_lines.append("%s: %s" % (key, value))

        response_lines.append("")
        return response_lines

    def write(self, data):
        """Write all the data of the response.
           Can be called just once.
        """
        if self._request_started:
            print "Request has already written a response:"
            traceback.print_stack()
            return

        data = self.convert(self, data)

        self._outgoing_headers['content-length'] = len(data)

        response_lines = self.begin_response(len(data))
        response_lines.append(data)
        self.protocol.wfile.write("\r\n".join(response_lines))
    push = write

    def write_chunked(self, data):
        """Write a chunk of the response.
           Can be called multiple times.
        """
        if not self._request_started:
            self._chunked = True
            self.set_header('transfer-coding', 'chunked')
            response_lines = self.begin_response()
        else:
            if not self._chunked:
                print "Request has already started without chunking:"
                traceback.print_stack()
                return
            response_lines = []

        data = self.convert(self, data)

        response_lines.append("%X" % len(data))
        response_lines.append(data)
        response_lines.append("")
        self.protocol.wfile.write("\r\n".join(response_lines))

    def convert(self, second_self, data):
        return data

    def method(self):
        return self._method.lower()

    def path(self):
        return self._path
    uri = path

    def get_headers(self):
        return self._incoming_headers

    def get_header(self, header_name, default=None):
        return self.get_headers().get(header_name.lower(), default)

    def get_query_pairs(self):
        if not hasattr(self, '_split_query'):
            if self._query is None:
                self._split_query = ()
            else:
                spl = self._query.split('&')
                spl = [x.split('=', 1) for x in spl if x]
                self._split_query = []
                for query in spl:
                    if len(query) == 1:
                        key = query[0]
                        value = ''
                    else:
                        key, value = query
                    self._split_query.append((urllib.unquote_plus(key), urllib.unquote_plus(value)))

        return self._split_query

    def get_queries_generator(self, name):
        """Generate all query parameters matching the given name.
        """
        for key, value in self.get_query_pairs():
            if key == name or not name:
                yield value

    get_queries = lambda self, name: list(self.get_queries_generator)

    def get_query(self, name, default=None):
        try:
            return self.get_queries_generator(name).next()
        except StopIteration:
            return default

    def get_arg_list(self, name):
        return self.get_field_storage().getlist(name)

    def get_arg(self, name, default=None):
        return self.get_field_storage().getfirst(name, default)

    def get_field_storage(self):
        if not hasattr(self, '_field_storage'):
            if self.method() == 'get':
                data = ''
            if self._query:
                data = self._query
                fl = cStringIO.StringIO(data)
            else:
                fl = self.protocol.rfile
            ## Allow our resource to provide the FieldStorage instance for
            ## customization purposes.
            headers = self.get_headers()
            environ = dict(
                REQUEST_METHOD='POST',
                QUERY_STRING=self._query or '')
            if (hasattr(self, 'resource') and
                hasattr(self.resource, 'getFieldStorage')):
                self._field_storage = self.resource.getFieldStorage(
                    self, fl, headers, environ)
            else:
                self._field_storage = cgi.FieldStorage(
                    fl, headers, environ=environ)

        return self._field_storage

    def set_header(self, key, value):
        if key.lower() == 'connection' and value.lower() == 'close':
            self.protocol.close_connection = 1
        self._outgoing_headers[key.lower()] = value
    __setitem__ = set_header

    def get_outgoing_header(self, key):
        return self._outgoing_headers[key.lower()]

    def has_outgoing_header(self, key):
        return self._outgoing_headers.has_key(key.lower())

    def socket(self):
        return self.protocol.socket

    def read_body(self):
        if hasattr(self, '_cached_body'):
            return self._cached_body
        length = self.get_header('content-length')
        if length:
            length = int(length)
        if length:
            self._cached_body = self.socket().read(length)
        else:
            self._cached_body = ''
        return self._cached_body

    def __repr__(self):
        return "<Request %s %s>" % (
            getattr(self, '_method'), getattr(self, '_path'))


class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
    protocol_version = "HTTP/1.0"
    def __init__(self, request, client_address, server):
        self.socket = self.request = self.rfile = self.wfile = request
        self.client_address = client_address
        self.server = server
        self._code = 200
        self._message = 'OK'

    def set_response_code(self, request, code, message):
        self._code = code
        self._message = message

    def generate_status_line(self):
        return [
            "%s %d %s" % (
                self.protocol_version, self._code, self._message)]

    def handle(self):
        self.close_connection = 0

        while not self.close_connection:
            self.raw_requestline = self.rfile.readline()
        
            if not self.raw_requestline:
                self.close_connection = 1
                return
            if not self.parse_request():
                return
            request = Request(self, self.command, self.path, self.headers)
            request.set_header('Server', self.version_string())
            request.set_header('Date', self.date_time_string())
            
            try:
                self.server.site.handle_request(request)
            except socket.error, e:
                # Broken pipe, connection reset by peer
                if e[0] in (32, 54):
                    #print "Remote host closed connection before response could be sent"
                    pass
                else:
                    raise
            except Exception, e:
                request.response(500)
                request.write('Internal Server Error')
                self.socket.close()
                raise
        self.socket.close()


class Server(BaseHTTPServer.HTTPServer):
    def __init__(self, socket, address, site, log, args, kw):
        self.socket = socket
        self.address = address
        self.site = site
        if log:
            self.log = log
            log.write = log.info
        else:
            self.log = self
        self.arguments = args
        self.__dict__.update(kw)

    def write(self, something):
        sys.stdout.write('%s\n' % (something, ))

    def log_message(self, message):
        self.log.write(message)

    def log_exception(self, type, value, tb):
        print ''.join(traceback.format_exception(type, value, tb))

    def write_access_log_line(self, *args):
        """Write a line to the access.log. Arguments:
        client_address, date_time, requestline, code, size, request_time
        """
        self.log.write(
            '%s - - [%s] "%s" %s %s %s' % args)


def server(sock, site, log=None, *args, **kw):
    serv = Server(sock, sock.getsockname(), site, log, args, kw)
    try:
        print "httpd starting up on", sock.getsockname()
        while True:
            try:
                new_sock, address = sock.accept()
                proto = HttpProtocol(new_sock, address, serv)
                api.spawn(proto.handle)
            except KeyboardInterrupt:
                api.get_hub().remove_descriptor(sock.fileno())
                print "httpd exiting"
                break
    finally:
        try:
            sock.close()
        except socket.error:
            pass





import os
import fcntl
import socket
import errno
from errno import EWOULDBLOCK

try:
    from OpenSSL import SSL
except ImportError:
    class SSL(object):
        class WantWriteError(object):
            pass

        class WantReadError(object):
            pass

        class ZeroReturnError(object):
            pass


def g_log(*args):
    import sys
    import greenlet
    from eventlet.greenlib import greenlet_id
    g_id = greenlet_id()
    if g_id is None:
        if greenlet.getcurrent().parent is None:
            ident = 'greenlet-main'
        else:
            g_id = id(greenlet.getcurrent())
            if g_id < 0:
                g_id += 1 + ((sys.maxint + 1) << 1)
            ident = '%08X' % (g_id,)
    else:
        ident = 'greenlet-%d' % (g_id,)
    print >>sys.stderr, '[%s] %s' % (ident, ' '.join(map(str, args)))

CONNECT_ERR = (errno.EINPROGRESS, errno.EALREADY, EWOULDBLOCK)
CONNECT_SUCCESS = (0, errno.EISCONN)
def socket_connect(descriptor, address):
    err = descriptor.connect_ex(address)
    if err in CONNECT_ERR:
        return None
    if err not in CONNECT_SUCCESS:
        raise socket.error(err, errno.errorcode[err])
    return descriptor

__original_socket__ = socket.socket

def tcp_socket():
    s = __original_socket__(socket.AF_INET, socket.SOCK_STREAM)
    set_nonblocking(s)
    return s


__original_ssl__ = socket.ssl


def wrap_ssl(sock, certificate=None, private_key=None):
    from OpenSSL import SSL
    from eventlet import wrappedfd, util
    context = SSL.Context(SSL.SSLv23_METHOD)
    print certificate, private_key
    if certificate is not None:
        context.use_certificate_file(certificate)
    if private_key is not None:
        context.use_privatekey_file(private_key)

    return wrappedfd.wrapped_fd(
        SSL.Connection(context, sock))


def wrap_socket_with_coroutine_socket():
    def new_socket(*args, **kw):
        from eventlet import wrappedfd
        s = __original_socket__(*args, **kw)
        set_nonblocking(s)
        return wrappedfd.wrapped_fd(s)
    socket.socket = new_socket

    socket.ssl = wrap_ssl


def socket_bind_and_listen(descriptor, addr=('', 0), backlog=50):
    set_reuse_addr(descriptor)
    descriptor.bind(addr)
    descriptor.listen(backlog)
    return descriptor
    
def socket_accept(descriptor):
    try:
        return descriptor.accept()
    except socket.error, e:
        if e[0] == EWOULDBLOCK:
            return None
        raise

def socket_send(descriptor, data):
    try:
        return descriptor.send(data)
    except socket.error, e:
        if e[0] == EWOULDBLOCK:
            return 0
        raise
    except SSL.WantWriteError:
        return 0

# winsock sometimes throws ENOTCONN
SOCKET_CLOSED = (errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN)
def socket_recv(descriptor, buflen):
    try:
        return descriptor.recv(buflen)
    except socket.error, e:
        if e[0] == EWOULDBLOCK:
            return None
        if e[0] in SOCKET_CLOSED:
            return ''
        raise
    except SSL.WantReadError:
        return None
    except SSL.ZeroReturnError:
        return ''

def set_reuse_addr(descriptor):
    try:
        descriptor.setsockopt(
            socket.SOL_SOCKET,
            socket.SO_REUSEADDR,
            descriptor.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR) | 1,
        )
    except socket.error:
        pass
    
def set_nonblocking(descriptor):
    if hasattr(descriptor, 'setblocking'):
        # socket
        descriptor.setblocking(0)
    else:
        # fd
        if hasattr(descriptor, 'fileno'):
            fd = descriptor.fileno()
        else:
            fd = descriptor
        flags = fcntl.fcntl(fd, fcntl.F_GETFL)
        fcntl.fcntl(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)
    return descriptor





import os
import time
import socket
import os.path

import simplejson

import mu

from eventlet import channel
from eventlet import timer
from eventlet import api


class TimedOut(Exception):
    pass


class DumbEncoder(simplejson.JSONEncoder):
    def default(self, obj):
        return str(obj)

class Target(mu.Resource):
    debug = False
    cache = False
    contentType = 'text/xml'
    
    reconnect_timer = None
    channel = None

    def __init__(self, target_id, resource, peer_name):
        self.target_id = target_id
        self.queue = []
        self.resource = resource
        self.peer_name = peer_name

    def handle(self, request):
        try:
            mu.Resource.handle(self, request)
        except socket.error, (errno, reason):
            if errno == 32:
                self.reconnect_timer.cancel()
                return
            raise

    def get_default(self):
        return "()"

    def handle_post(self, req):
        req.set_header('pragma', 'no-cache')
        req.set_header('content-type', 'text/xml')
        if self.reconnect_timer:
            # The client made another request within 5 seconds, horray!
            self.reconnect_timer.cancel()
        req.resource = self.resource

        sock = req.protocol.socket
        def bounce(descriptor=None):
            try:
                api.get_hub().remove_descriptor(sock.fileno())
            except socket.error:
                ## server closed;
                ## ignore
                return
            # Client closed browser; remove ourself
            # this will be done in socket.error in handle above
            self._client_timeout()
            
        api.get_hub().add_descriptor(sock.fileno(), read=lambda *a: None, exc=bounce)

        got_event = dict(
            request_id=self.request_id,
            target_id=self.target_id,
            events=self._process_events())

        # cancel the close watchdog
        api.get_hub().remove_descriptor(sock.fileno())

        req.write(mu.xml(simplejson.dumps(got_event, cls=DumbEncoder)))
        return None

    def _process_events(self):
        if self.queue:
            queue, self.queue = self.queue, []
            return queue

        # Server has 25 seconds to send an event before being disconnected
        self.timer = timer.Timer(25, self._server_timeout)
        self.timer.schedule()
        self.channel = channel.channel()
        try:
            event = self.channel.receive()
        except TimedOut:
            event = None
        self.timer.cancel()
        self.channel = None

        # Client has 5 seconds to reconnect before being disconnected
        self.reconnect_timer = timer.Timer(5, self._client_timeout)
        self.reconnect_timer.schedule()

        if event != None:
            return [event]
        return []

    def _send(self, event):
        if self.channel is not None:
            self.channel.send(event)
        else:
            self.queue.append(event)    

    def _server_timeout(self):
        # We didn't have an event on the server; send a noop
        self.channel.send_exception(TimedOut)

    def _client_timeout(self):
        # Client didn't reconnect fast enough; we're going bye-bye
        self.resource._event_target_disconnected(self)

    def msg(self, transform, data, broadcasted=False, log=True):
        """Send a message to this target.

        transform: The client-side function to invoke.
        data: The data to send as the argument to the function.
        """
        to_send = (transform, simplejson.dumps(data, cls=DumbEncoder))
        if log and not broadcasted:
            console = self.resource.console
            if console is not None:
                console.log_transform(to_send)
        self._send(to_send)

    def peer_name_string(self):
        return "%s-%s" % self.peer_name


class Router(mu.Resource):
    debug = True
    transforms = 'transforms.js'
    stylesheet = 'stylesheet.css'
    template = mu.xml("""<html><head>
      <script src="http://www.mochikit.com/MochiKit/MochiKit.js"> </script>
      <script src="eventrouter.js" /> </script>
      <script src="transforms.js"> </script>
      <link href="styles.css" rel="stylesheet" />
    </head><body id="body">
      <div id="contents">
        Contents here.
      </div>
      <div id="bottom"> </div>
    </body></html>""")

    def __init__(self, transforms=None, stylesheet=None, console=None):
        if console is True:
            self.console = self
        else:
            self.console = console
        self.clients = dict()
        self.targets_by_id = dict()
        self.targets_by_target_id = dict()
        self.targets = list()
        base = os.path.split(__file__)[0]
        self.child_mochi = mu.Directory(
            os.path.join(base, '../../js/MochiKit/'))
        if transforms is not None:
            self.transforms = transforms
        if stylesheet is not None:
            self.stylesheet = stylesheet
        if os.path.exists(self.transforms):
            setattr(self, 'child_transforms.js', mu.File(self.transforms))
        if os.path.exists(self.stylesheet):
            setattr(self, 'child_styles.css', mu.File(self.stylesheet))
        setattr(self, 'child_eventrouter.js', mu.File(os.path.join(base, 'eventrouter.js')))

    def child_(self, req, segment):
        print "child underscore", req, segment
        return self

    def broadcast(self, transform, data, log=True):
        """Send a message to all clients connected to this router.

        transform: The client-side function to invoke on all connected browsers.
        data: The data to send to the function.
        """
        for client in self.clients.values():
            if log and self.console is not None:
                self.console.log_transform((transform, simplejson.dumps(data)))
                broadcasted = True
            else:
                broadcasted = False
            client.msg(transform, data, broadcasted=broadcasted, log=log)

    def _event_target_connected(self, target):
        collector = []
        def new_msg(name, value):
            collector.append((name, value))

        target.msg = new_msg
        result = self.event_connect(target)
        del target.msg

        target.msg('__initialize__', result)
        for name, value in collector:
            target.msg(name, value)

        self.targets.append(target)

    def event_connect(self, target):
        """override me
        """
        pass

    def _event_target_disconnected(self, target):
        if target in self.targets:
            self.targets.remove(target)
        else:
            ## This can go away if this bug goes away.
            print "Bug tracking: Target was not present", target
        self.targets_by_id.pop(str(id(target)))
        self.targets_by_target_id.pop(target.target_id)
        self.clients.pop(target.request_id)
        self.event_disconnect(target)

    def event_disconnect(self, target):
        """override me
        """
        pass

    def child_events(self, req, name):
        old_request_id = req.get_query('request_id')
        if old_request_id in self.clients:
            reconnect = True
            target = self.clients.pop(old_request_id)
        else:
            if old_request_id:
                print "WTF. This is a bug.", old_request_id, self.clients
            reconnect = False
            self.target_id = getattr(self, 'target_id', 0) + 1
            target = Target(self.target_id, self, req.socket().getpeername())
            self._event_target_connected(target)

        ## TODO: If this proves to fix the bug, just make the request_id always time() + target_id
        target.request_id = str(time.time())
        if target.request_id in self.clients:
            print "Bug tracking: Duplicate request_id", target.request_id

        target.request_id += "-" + str(self.target_id)
        self.clients[target.request_id] = target
        self.targets_by_id[str(id(target))] = target
        self.targets_by_target_id[target.target_id] = target
        if not reconnect:
            self.event_connect(target)
        req.target = target
        return target

    def handle_get(self, req):
        print "handle_get", req, req.path()
        if not req.path().endswith('/'):
            # redirect to url with trailing slash added
            req.response(301, 'Moved Permanently')
            req.set_header('location', 'http://' + req.socket().getsockname()[0] + ':12040' + req.path() + '/')
            req.write('')
            return
        return mu.Resource.handle_get(self, req)

    def handle_post(self, req):
        post = req.get_query('post', 'default')
        target_id = int(req.get_query('target_id'))
        body = simplejson.loads(req.read_body())
        if self.console is not None:
            self.console.log_send_data((post, str(self.targets_by_target_id[target_id]), body))
        handler = getattr(self, 'post_%s' % (post, ), None)
        if handler is None:
            print "No post_* handler for send_data:", post
        else:
            handler(req, self.targets_by_target_id[target_id], body)
        req.write('')



"""test_api
"""

import unittest

from eventlet import api, wrappedfd, util
import socket


def check_hub():
    # Clear through the descriptor queue
    api.sleep(0)
    api.sleep(0)
    assert not api.get_hub().descriptors, repr(api.get_hub().descriptors)
    # Stop the runloop
    api.get_hub().runloop.abort()
    api.sleep(0)
    assert not api.get_hub().runloop.running


class TestApi(unittest.TestCase):
    def test_tcp_listener(self):
        socket = api.tcp_listener(('0.0.0.0', 0))
        assert socket.getsockname()[0] == '0.0.0.0'
        socket.close()

        check_hub()

    def dont_test_connect_tcp(self):
        """This test is broken. Please name it test_connect_tcp and fix
        the bug (or the test) so it passes.
        """
        def accept_once(listenfd):
            try:
                conn, addr = listenfd.accept()
                conn.write('hello\n')
                conn.close()
            finally:
                listenfd.close()

        server = api.tcp_listener(('0.0.0.0', 0))
        api.spawn(accept_once, server)

        client = api.connect_tcp(('127.0.0.1', server.getsockname()[1]))
        assert client.readline() == 'hello\n'

        assert client.read() == ''
        client.close()

        check_hub()

    def test_server(self):
        server = api.tcp_listener(('0.0.0.0', 0))
        bound_port = server.getsockname()[1]
        connected = []

        def accept_twice((conn, addr)):
            print 'connected'
            connected.append(True)
            conn.close()
            if len(connected) == 2:
                server.close()

        api.call_after(0, api.connect_tcp, ('127.0.0.1', bound_port))
        api.call_after(0, api.connect_tcp, ('127.0.0.1', bound_port))
        api.tcp_server(server, accept_twice)

        assert len(connected) == 2

        check_hub()

    def dont_test_trampoline_timeout(self):
        """This test is broken. Please change it's name to test_trampoline_timeout,
        and fix the bug (or fix the test)
        """
        server = api.tcp_listener(('0.0.0.0', 0))
        bound_port = server.getsockname()[1]

        try:
            desc = wrappedfd.wrapped_fd(util.tcp_socket())
            api.trampoline(desc, read=True, write=True, timeout=0.1)
        except api.TimeoutError:
            pass # test passed
        else:
            assert False, "Didn't timeout"

        check_hub()

    def test_timeout_cancel(self):
        server = api.tcp_listener(('0.0.0.0', 0))
        bound_port = server.getsockname()[1]

        def client_connected((conn, addr)):
            conn.close()

        def go():
            client = util.tcp_socket()

            desc = wrappedfd.wrapped_fd(client)
            desc.connect(('127.0.0.1', bound_port))
            try:
                api.trampoline(desc, read=True, write=True, timeout=0.1)
            except api.TimeoutError:
                assert False, "Timed out"

            server.close()
            client.close()

        api.call_after(0, go)

        api.tcp_server(server, client_connected)

        check_hub()

    def dont_test_explicit_hub(self):
        """This test is broken. please change it's name to test_explicit_hub
        and make it pass (or fix the test)
        """
        api.use_hub(Foo)
        assert isinstance(api.get_hub(), Foo), api.get_hub()

        api.use_hub(api.get_default_hub())

        check_hub()


class Foo(object):
    pass


if __name__ == '__main__':
    unittest.main()




import sys
import itertools

import greenlet

from eventlet import tls

__all__ = [
    'switch', 'kill', 'tracked_greenlets',
    'greenlet_id', 'greenlet_dict', 'GreenletContext',
    'tracked_greenlet',
]

try:
    reversed
except NameError:
    def reversed(something):
        for x in something[::-1]:
            yield x

_threadlocal = tls.local()

def tracked_greenlet():
    """
    Returns a greenlet that has a greenlet-local dictionary and can be
    used with GreenletContext and enumerated with tracked_greenlets
    """
    return greenlet.greenlet(greenlet_body)

class GreenletContextManager(object):
    """
    Per-thread manager for GreenletContext.  Created lazily on registration
    """
    def __new__(cls, *args, **kw):
        dct = greenlet_dict()
        self = dct.get('greenlet_context', None)
        if self is not None:
            return self
        self = super(GreenletContextManager, cls).__new__(cls, *args, **kw)
        dct['greenlet_context'] = self
        self.contexts = []
        return self

    def add_context(self, ctx):
        fn = getattr(ctx, '_swap_in', None)
        if fn is not None:
            fn()
        self.contexts.append(ctx)

    def remove_context(self, ctx):
        try:
            idx = self.contexts.index(ctx)
        except ValueError:
            return
        else:
            del self.contexts[idx]
        fn = getattr(ctx, '_swap_out', None)
        if fn is not None:
            fn()
        fn = getattr(ctx, '_finalize', None)
        if fn is not None:
            fn()

    def swap_in(self):
        for ctx in self.contexts:
            fn = getattr(ctx, '_swap_in', None)
            if fn is not None:
                fn()

    def swap_out(self):
        for ctx in reversed(self.contexts):
            fn = getattr(ctx, '_swap_out', None)
            if fn is not None:
                fn()

    def finalize(self):
        for ctx in reversed(self.contexts):
            fn = getattr(ctx, '_swap_out', None)
            if fn is not None:
                fn()
            fn = getattr(ctx, '_finalize', None)
            if fn is not None:
                fn()
        del self.contexts[:]
        try:
            del greenlet_dict()['greenlet_context']
        except KeyError:
            pass

class GreenletContext(object):
    """
    A context manager to be triggered when a specific tracked greenlet is
    swapped in, swapped out, or finalized.

    To use, subclass and override the swap_in, swap_out, and/or finalize
    methods, for example::
        
        import greenlib
        from greenlib import greenlet_id, tracked_greenlet, switch

        class NotifyContext(greenlib.GreenletContext):

            def swap_in(self):
                print "swap_in"

            def swap_out(self):
                print "swap_out"

            def finalize(self):
                print "finalize"

        def another_greenlet():
            print "another_greenlet"

        def notify_demo():
            print "starting"
            NotifyContext().register()
            switch(tracked_greenlet(), (another_greenlet,))
            print "finishing"
            # we could have kept the NotifyContext object
            # to unregister it here but finalization of all
            # contexts is implicit when the greenlet returns

        t = tracked_greenlet()
        switch(t, (notify_demo,))

    The output should be:

        starting
        swap_in
        swap_out
        another_greenlet
        swap_in
        finishing
        swap_out
        finalize
    
    """
    _balance = 0
    
    def _swap_in(self):
        if self._balance != 0:
            raise RuntimeError("balance != 0: %r" % (self._balance,))
        self._balance = self._balance + 1
        fn = getattr(self, 'swap_in', None)
        if fn is not None:
            fn()

    def _swap_out(self):
        if self._balance != 1:
            raise RuntimeError("balance != 1: %r" % (self._balance,))
        self._balance = self._balance - 1
        fn = getattr(self, 'swap_out', None)
        if fn is not None:
            fn()

    def register(self):
        GreenletContextManager().add_context(self)

    def unregister(self):
        GreenletContextManager().remove_context(self)

    def _finalize(self):
        fn = getattr(self, 'finalize', None)
        if fn is not None:
            fn()


def kill(g):
    """
    Kill the given greenlet if it is alive by sending it a GreenletExit.
    
    Note that of any other exception is raised, it will pass-through!
    """
    if not g:
        return
    kill_exc = greenlet.GreenletExit()
    try:
        try:
            g.parent = greenlet.getcurrent()
        except ValueError:
            pass
        switch(g, exc=kill_exc)
    except greenlet.GreenletExit, e:
        if e is not kill_exc:
            raise

def tracked_greenlets():
    """
    Return a list of greenlets tracked in this thread.  Tracked greenlets
    use greenlet_body() to ensure that they have greenlet-local storage.
    """
    try:
        return _threadlocal.greenlets.keys()
    except AttributeError:
        return []

def greenlet_id():
    """
    Get the id of the current tracked greenlet, returns None if the
    greenlet is not tracked.
    """
    try:
        d = greenlet_dict()
    except RuntimeError:
        return None
    return d['greenlet_id']

def greenlet_dict():
    """
    Return the greenlet local storage for this greenlet.  Raises RuntimeError
    if this greenlet is not tracked.
    """
    self = greenlet.getcurrent()
    try:
        return _threadlocal.greenlets[self]
    except (AttributeError, KeyError):
        raise RuntimeError("greenlet %r is not tracked" % (self,))

def _greenlet_context(dct=None):
    if dct is None:
        try:
            dct = greenlet_dict()
        except RuntimeError:
            return None
    return dct.get('greenlet_context', None)

def _greenlet_context_call(name, dct=None):
    ctx = _greenlet_context(dct)
    fn = getattr(ctx, name, None)
    if fn is not None:
        fn()

def greenlet_body(value, exc):
    """
    Track the current greenlet during the execution of the given callback,
    normally you would use tracked_greenlet() to get a greenlet that uses this.

    Greenlets using this body must be greenlib.switch()'ed to
    """
    if exc is not None:
        if isinstance(exc, tuple):
            raise exc[0], exc[1], exc[2]
        raise exc
    cb, args = value[0], value[1:]
    try:
        greenlets = _threadlocal.greenlets
    except AttributeError:
        greenlets = _threadlocal.greenlets = {}
    else:
        if greenlet.getcurrent() in greenlets:
            raise RuntimeError("greenlet_body can not be called recursively!")
    try:
        greenlet_id = _threadlocal.next_greenlet_id.next()
    except AttributeError:
        greenlet_id = 1
        _threadlocal.next_greenlet_id = itertools.count(2)
    greenlets[greenlet.getcurrent()] = {'greenlet_id': greenlet_id}
    try:
        return cb(*args)
    finally:
        _greenlet_context_call('finalize')
        greenlets.pop(greenlet.getcurrent(), None)
       
def switch(other=None, value=None, exc=None):
    """
    Switch to another greenlet, passing value or exception
    """
    self = greenlet.getcurrent()
    if other is None:
        other = self.parent
    if other is None:
        other = self
    if not (other or hasattr(other, 'run')):
        raise RuntimeError("Switching to dead greenlet %r" % (other,))
    _greenlet_context_call('swap_out')
    try:
        rval = other.switch(value, exc)
        if not rval or not other:
            res, exc = rval, None
        else:
            res, exc = rval
    except:
        res, exc = None, sys.exc_info()
    _greenlet_context_call('swap_in')
    if isinstance(exc, tuple):
        typ, exc, tb = exc
        raise typ, exc, tb
    elif exc is not None:
        raise exc
    return res




import os.path


from mulib import eventrouter


class Console(eventrouter.Router):
    transforms = os.path.join(os.path.dirname(__file__), 'logconsole.js')
    stylesheet = os.path.join(os.path.dirname(__file__), 'logconsole.css')

    def __init__(self, my_ip, *args, **kw):
        self.my_ip = my_ip
        eventrouter.Router.__init__(self, *args, **kw)

    def event_connect(self, target):
        return "logconsole running on %s" % (self.my_ip, )

    def emit(self, output):
        self.broadcast('log', output, log=False)

    def log(self, *message):
        output = ' '.join([str(x) for x in message])
        self.broadcast('log', output, log=False)

    def log_request(self, request_id, method, netloc, path, headers, body):
        self.broadcast('log-request', {'request-id': request_id, 'method': method, 'url': netloc, 'path': path, 'headers': headers, 'body': body}, log=False)

    def log_response(self, request_id, status, headers, body):
        self.broadcast('log-response', {'request-id': request_id, 'status': status, 'headers': headers, 'body': body}, log=False)

    def log_transform(self, (transform, data)):
        self.broadcast('log-transform', (transform, data), log=False)

    def log_send_data(self, (post, target, body)):
        self.broadcast('log-send-data', (post, target, body), log=False)






import sys
import traceback

from eventlet import api
from eventlet import timer

from twisted.internet import posixbase
from twisted.internet.interfaces import IReactorFDSet

try:
    from zope.interface import implements
    _working = True
except ImportError:
    _working = False
    def implements(*args, **kw):
        pass


class TwistedTimer(object):
    def __init__(self, timer):
        self.timer = timer

    def cancel(self):
        self.timer.cancel()

    def getTime(self):
        return self.timer.scheduled_time

    def delay(self, seconds):
        hub = api.get_hub()
        new_time = hub.clock() - self.timer_scheduled_time + seconds
        self.timer.cancel()
        cb, args, kw = self.timer.tpl
        self.timer = hub.schedule_call(new_time, cb, *args, **kw)

    def reset(self, new_time):
        self.timer.cancel()
        cb, args, kw = self.timer.tpl
        self.timer = api.get_hub().schedule_call(new_time, cb, *args, **kw)

    def active(self):
        return not self.timer.called


class EventletReactor(posixbase.PosixReactorBase):
    implements(IReactorFDSet)

    def __init__(self, *args, **kw):
        self._readers = {}
        self._writers = {}
        posixbase.PosixReactorBase.__init__(self, *args, **kw)

    def callLater(self, func, *args, **kw):
        return TwistedTimer(api.call_after(func, *args, **kw))

    def run(self):
        self.running = True
        self._stopper = api.call_after(sys.maxint / 1000.0, lambda: None)
        ## schedule a call way in the future, and cancel it in stop?
        api.get_hub().runloop.run()

    def stop(self):
        self._stopper.cancel()
        posixbase.PosixReactorBase.stop(self)
        api.get_hub().remove_descriptor(self._readers.keys()[0])
        api.get_hub().runloop.abort()

    def addReader(self, reader):
        fileno = reader.fileno()
        self._readers[fileno] = reader
        api.get_hub().add_descriptor(fileno, read=self._got_read)

    def _got_read(self, fileno):
        self._readers[fileno].doRead()

    def addWriter(self, writer):
        fileno = writer.fileno()
        self._writers[fileno] = writer
        api.get_hub().add_descriptor(fileno, write=self._got_write)

    def _got_write(self, fileno):
        self._writers[fileno].doWrite()

    def removeReader(self, reader):
        fileno = reader.fileno()
        if fileno in self._readers:
            self._readers.pop(fileno)
        api.get_hub().remove_descriptor(fileno)

    def removeWriter(self, writer):
        fileno = writer.fileno()
        if fileno in self._writers:
            self._writers.pop(fileno)
        api.get_hub().remove_descriptor(fileno)

    def removeAll(self):
        return self._removeAll(self._readers.values(), self._writers.values())


def emulate():
    if not _working:
        raise RuntimeError, "Can't use twistedsupport because zope.interface is not installed."
    reactor = EventletReactor()
    from twisted.internet.main import installReactor
    installReactor(reactor)


__all__ = ['emulate']






import code
import os.path
import popen2
import sys
import time

from eventlet import api
from eventlet import logutil

from mulib import mu
from mulib import eventrouter

import simplejson

from indra import llsd


class ShellConsole(eventrouter.Router):
    transforms = os.path.join(os.path.dirname(__file__), 'shellconsole.js')
    stylesheet = os.path.join(os.path.dirname(__file__), 'shellconsole.css')

    def __init__(self, my_host, root):
        self.my_host = my_host
        self.interactor = code.InteractiveConsole(root)
        self.logsucker = logutil.LineLogger(self.emit)
        self.reset_collector()
        self.partial_lines = ''
        self.ps1_prompt = '>>> '
        self.ps2_prompt = '... '
        self.prompt = self.ps1_prompt
        eventrouter.Router.__init__(self)

    def event_connect(self, target):
        return [target.peer_name_string(), "shellconsole running on %s" % (self.my_host, )]

    def reset_collector(self):
        self.collector = []

    def emit(self, line):
        self.collector.append(line)

    def post_key_typed(self, data):
        my_name = data['my_name']
        current_input = data['current_input']
        for target in self.targets:
            if target.peer_name_string() != my_name:
                target.msg('key-typed', [my_name, current_input])

    def post_console_input(self, data_map):
        self.reset_collector()
        command_id = time.time()
        cmd = str(data_map['input'])
        print "Running command", command_id, cmd
        self.broadcast('input-line', [data_map['my_name'], self.prompt, cmd])
        stderr = self.saved_stderr = sys.stderr
        stdout = self.saved_stdout = sys.stdout

        try:
            sys.stderr = self.logsucker
            sys.stdout = self.logsucker
            result = self.interactor.runsource(self.partial_lines + cmd)
        finally:
            sys.stderr, sys.stdout = self.saved_stderr, self.saved_stdout
        if not result:
            self.prompt = self.ps1_prompt
            self.partial_lines = ''
        else:
            self.prompt = self.ps2_prompt
            self.partial_lines += cmd + '\n'
        for output_line in self.collector:
            print output_line
            self.broadcast('output-line', output_line)
        print "Finished shell command", `result`, command_id

    def handle_post(self, req):
        action = req.get_query('post', 'default')
        data = req.read_body()
        data = simplejson.loads(data)
        meth = getattr(self, 'post_' + action, None)(data)
        req.write(mu.xml(''))




import threading
import weakref

__all__ = ['local']

class _local(object):
    """
    Crappy Python 2.3 compatible implementation of thread-local storage
    """

    __slots__ = ('__thread_dict__',)

    def __init__(self):
        object.__setattr__(self, '__thread_dict__', weakref.WeakKeyDictionary())
        
    def __getattr__(self, attr):
        try:
            return self.__thread_dict__[threading.currentThread()][attr]
        except KeyError:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        t = threading.currentThread()
        try:
            d = self.__thread_dict__[t]
        except KeyError:
            d = self.__thread_dict__[t] = {}
        d[attr] = value

try:
    local = threading.local
except AttributeError:
    local = _local



import time

__all__ = ['format_date_time']

# Weekday and month names for HTTP date/time formatting; always English!
_weekdayname = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
_monthname = [None, # Dummy so we can use 1-based month numbers
              "Jan", "Feb", "Mar", "Apr", "May", "Jun",
              "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]

def format_date_time(timestamp):
    year, month, day, hh, mm, ss, wd, y, z = time.gmtime(timestamp)
    return "%s, %02d %3s %4d %02d:%02d:%02d GMT" % (
        _weekdayname[wd], day, _monthname[month], year, hh, mm, ss
    )



import sys
import socket
import linecache
import inspect

import greenlet

from eventlet import greenlib, tls

__all__ = [
    'use_hub', 'get_hub', 'sleep', 'spawn', 'kill',
    'call_after', 'trampoline', 'tcp_listener', 'tcp_server',
]


class TimeoutError(Exception):
    pass

_threadlocal = tls.local()

def tcp_listener(address):
    """
    Listen on the given (ip, port) address with a TCP socket.
    Returns a socket object which one should call accept() on to
    accept a connection on the newly bound socket.

    Generally, the returned socket will be passed to tcp_server,
    which accepts connections forever and spawns greenlets for
    each incoming connection.
    """
    from eventlet import wrappedfd, util
    socket = wrappedfd.wrapped_fd(util.tcp_socket())
    util.socket_bind_and_listen(socket, address)
    return socket

def ssl_listener(address, certificate, private_key):
    """Listen on the given (ip, port) address with a TCP socket that
    can do SSL.

    Returns a socket object which one should call accept() on to
    accept a connection on the newly bound socket.

    Generally, the returned socket will be passed to tcp_server,
    which accepts connections forever and spawns greenlets for
    each incoming connection.
    """
    from eventlet import util
    socket = util.wrap_ssl(util.tcp_socket(), certificate, private_key)
    util.socket_bind_and_listen(socket, address)

    return socket

def connect_tcp(address):
    """
    Create a TCP connection to address (host, port) and return the socket.
    """
    from eventlet import wrappedfd, util
    desc = wrappedfd.wrapped_fd(util.tcp_socket())
    desc.connect(address)
    return desc

def tcp_server(listensocket, server, *args, **kw):
    """
    Given a socket, accept connections forever, spawning greenlets
    and executing "server" for each new incoming connection.
    When listensocket is closed, the tcp_server greenlet will end.

    listensocket:
        The socket to accept connections from.

    server:
        The callable to call when a new connection is made.

    *args:
        The arguments to pass to the call to server.

    **kw:
        The keyword arguments to pass to the call to server.
    """
    try:
        try:
            while True:
                spawn(server, listensocket.accept(), *args, **kw)
        except socket.error, e:
            # Broken pipe means it was shutdown
            if e[0] != 32:
                raise
    finally:
        listensocket.close()

def trampoline(fd, read=None, write=None, timeout=None):
    t = None
    hub = get_hub()
    self = greenlet.getcurrent()
    fileno = getattr(fd, 'fileno', lambda: fd)()
    def _do_close(fn):
        hub.remove_descriptor(fn)
        greenlib.switch(self, exc=socket.error(32, 'Broken pipe'))
    def _do_timeout(fn):
        hub.remove_descriptor(fn)
        greenlib.switch(self, exc=TimeoutError())
    def cb(_fileno):
        if t is not None:
            t.cancel()
        hub.remove_descriptor(fileno)
        greenlib.switch(self, fd)
    if timeout is not None:
        t = hub.schedule_call(timeout, _do_timeout)
    hub.add_descriptor(fileno, read and cb, write and cb, _do_close)
    return hub.switch()

def _spawn_startup(cb, args, kw, cancel=None):
    try:
        greenlib.switch(greenlet.getcurrent().parent)
        cancel = None
    finally:
        if cancel is not None:
            cancel()
    return cb(*args, **kw)

def _spawn(g):
    g.parent = greenlet.getcurrent()
    greenlib.switch(g)

def spawn(cb, *args, **kw):
    # killable
    t = None
    g = greenlib.tracked_greenlet()
    t = get_hub().schedule_call(0, _spawn, g)
    greenlib.switch(g, (_spawn_startup, cb, args, kw, t.cancel))
    return g

kill = greenlib.kill

def call_after(seconds, cb, *args, **kw):
    # cancellable
    def startup():
        g = greenlib.tracked_greenlet()
        greenlib.switch(g, (_spawn_startup, cb, args, kw))
        greenlib.switch(g)
    return get_hub().schedule_call(seconds, startup)

def get_default_hub():
    try:
        import eventlet.kqueuehub
    except ImportError:
        pass
    else:
        return eventlet.kqueuehub
    import select
    if hasattr(select, 'poll'):
        import eventlet.pollhub
        return eventlet.pollhub
    else:
        import eventlet.selecthub
        return eventlet.selecthub

def use_hub(mod=None):
    if mod is None:
        mod = get_default_hub()
    if hasattr(_threadlocal, 'hub'):
        del _threadlocal.hub
    if hasattr(mod, 'Hub'):
        _threadlocal.Hub = mod.Hub
    else:
        _threadlocal.Hub = mod

def get_hub():
    try:
        hub = _threadlocal.hub
    except AttributeError:
        try:
            _threadlocal.Hub
        except AttributeError:
            use_hub()
        hub = _threadlocal.hub = _threadlocal.Hub()
    return hub

def sleep(timeout=0):
    hub = get_hub()
    hub.schedule_call(timeout, greenlib.switch, greenlet.getcurrent())
    hub.switch()


def traceit(frame, event, arg):
    if event == 'line':
        lineno = frame.f_lineno
        if '__file__' in frame.f_globals:
            filename = frame.f_globals['__file__']
            if (filename.endswith('.pyc') or
                filename.endswith('.pyo')):
                filename = filename[:-1]
            name = frame.f_globals['__name__']
            line = linecache.getline(filename, lineno)
        else:
            name = '[unknown]'
            try:
                src = inspect.getsourcelines(frame)
                line = src[lineno]
            except IOError:
                line = 'Unknown code named [%s].  VM instruction #%d' % (
                    frame.f_code.co_name, frame.f_lasti)
        print '%s:%s: %s' % (name, lineno, line.rstrip())
    return traceit


def spew():
    sys.settrace(traceit)


def unspew():
    sys.settrace(None)




from eventlet.api import get_hub

class Timer(object):
    __slots__ = ['seconds', 'tpl', 'called', 'cancelled', 'scheduled_time']
    def __init__(self, seconds, cb, *args, **kw):
        """Create a timer.
            seconds: The minimum number of seconds to wait before calling
            cb: The callback to call when the timer has expired
            *args: The arguments to pass to cb
            **kw: The keyword arguments to pass to cb

        This timer will not be run unless it is scheduled in a runloop by
        calling timer.schedule() or runloop.add_timer(timer).
        """
        self.cancelled = False
        self.seconds = seconds
        self.tpl = cb, args, kw
        self.called = False

    def copy(self):
        cb, args, kw = self.tpl
        return self.__class__(self.seconds, cb, *args, **kw)
    
    def schedule(self):
        """Schedule this timer to run in the current runloop.
        """
        self.called = False
        self.scheduled_time = get_hub().runloop.add_timer(self)
        return self
        
    def __call__(self):
        if not self.called:
            self.called = True
            cb, args, kw = self.tpl
            cb(*args, **kw)
        
    def cancel(self):
        """Prevent this timer from being called. If the timer has already
        been called, has no effect.
        """
        self.cancelled = True
        self.called = True





import os
import os.path
import traceback

os.environ['PYTHON_EGG_CACHE'] = '/tmp/' + os.environ.get('USER', 'nobody')

if not os.path.exists(os.environ['PYTHON_EGG_CACHE']):
    os.mkdir(os.environ['PYTHON_EGG_CACHE'])


import sys
import syslog
import BaseHTTPServer


class NullServer(object):
    def __init__(self, site):
        self.site = site

    def log_message(self, message):
        pass
#        self.log.write(message)

    def write_access_log_line(self, *args):
        """Write a line to the access.log. Arguments:
        client_address, date_time, requestline, code, size, request_time
        """
        pass

    def log_exception(self, type, value, tb):
        print ''.join(traceback.format_exception(type, value, tb))


class CgiProtocol(object):
    request_version = 'HTTP/1.0'
    responses = BaseHTTPServer.BaseHTTPRequestHandler.responses
    def __init__(self, server):
        from eventlet import logutil

        self.server = server
        self.wfile = sys.stdout
        self.rfile = sys.stdin
        self.socket = sys.stdin
        sys.stdout, sys.stderr = (
            logutil.SysLogger(syslog.LOG_INFO),
            logutil.SysLogger(syslog.LOG_ERR))

        self.client_address = ('Unknown address - CGI', 'Unknown port')
        self.requestline = "Unknown request line - CGI"

    def set_response_code(self, request, code, message):
        request.set_header('Status', "%s %s" % (code, message))

    def generate_status_line(self):
        return []


def log_fatal_exception(msg="Internal Server Error"):
    from eventlet import logutil

    fatal_error_logger = logutil.SysLogger(syslog.LOG_ERR)
    formatted = ''.join(traceback.format_exception(*sys.exc_info()))
    fatal_error_logger.write(formatted)
    return "Status: 500 %s\r\nContent-type: text/plain\r\n\r\n%s" % (msg, formatted)


def run_as_cgi(module_name, class_name, *args, **keywords):
    """http://www.w3.org/CGI/
    http://hoohoo.ncsa.uiuc.edu/cgi/env.html
    http://hoohoo.ncsa.uiuc.edu/cgi/out.html
    """
    syslog.openlog(sys.argv[0], syslog.LOG_PID, syslog.LOG_LOCAL0)

    try:
        klass = getattr(__import__(module_name), class_name)
    except Exception:
        ## Catch import errors.
        print log_fatal_exception("cgiadapter could not import module")
        return

    try:
        site = mu.SiteMap(klass(*args, **keywords))
        server = NullServer(site)
        method = os.environ.get("REQUEST_METHOD", "GET")
        path = os.environ.get("PATH_INFO", "/")
        query = os.environ.get("QUERY_STRING", "")
        if query:
            path = "%s?%s" % (path, query)
        headers = {}
        for name in os.environ.keys():
            if name.startswith('HTTP_'):
                header_name = name[len('HTTP_'):].replace('_', '-')
                headers[header_name] = os.environ[name]
    except Exception:
        ## Catch exceptions in klass.__init__, and misc crap.
        print log_fatal_exception("cgiadapter error")

    from eventlet import httpd

    ## Theoretically we should never have an exception here...
    request = httpd.Request(CgiProtocol(server), method, path, headers)
    ## And if an exception happens inside here, mu will be sure to catch it.
    site.handle_request(request)

    if not request._request_started:
		request.write("Request completed without writing a response:\n%s\n%s" % (
			request, request.__dict__))




"""test_runloop
"""

import sys
import time
import StringIO
import unittest

from eventlet import runloop


class TestRunloop(unittest.TestCase):
    def test_empty(self):
        r = runloop.RunLoop()
        r.schedule_call(0, r.abort)
        r.run()
        assert not r.running


    def test_timer(self):
        r = runloop.RunLoop()
        r.schedule_call(0.125, r.abort)
        start_time = time.time()
        r.run()
        assert time.time() - start_time >= 0.125
        assert not r.running

    def test_observer(self):
        observed = []
        r = runloop.RunLoop()
        r.add_observer(lambda runloop, activity: observed.append(activity))
        r.schedule_call(0, r.abort)
        r.run()
        assert observed == ['entry', 'before_timers', 'before_waiting', 'after_waiting', 'exit']
        assert not r.running


    def test_remove_observer(self):
        r = runloop.RunLoop()

        observed = []
        def observe(runloop, mode):
            observed.append(mode)
            r.remove_observer(observe)

        looped = []
        def run_loop_twice(runloop, mode):
            if looped:
                r.abort()
            else:
                looped.append(True)

        r.add_observer(observe, 'before_timers')
        r.add_observer(run_loop_twice, 'after_waiting')
        r.run()
        assert len(observed) == 1
        assert not r.running

    def test_observer_exception(self):
        r = runloop.RunLoop()

        observed = []
        def observe(runloop, mode):
            observed.append(mode)
            raise Exception("Squelch me please")

        looped = []
        def run_loop_twice(runloop, mode):
            if looped:
                r.abort()
            else:
                looped.append(True)

        saved = sys.stderr
        sys.stderr = err = StringIO.StringIO()

        r.add_observer(observe, 'before_timers')
        r.add_observer(run_loop_twice, 'after_waiting')
        r.run()

        err.seek(0)
        sys.stderr = saved

        assert len(observed) == 1
        assert err.read()
        assert not r.running

    def test_timer_exception(self):
        r = runloop.RunLoop()

        observed = []
        def timer():
            observed.append(True)
            raise Exception("Squelch me please")

        looped = []
        def run_loop_twice(runloop, mode):
            if looped:
                r.abort()
            else:
                looped.append(True)

        saved = sys.stderr
        sys.stderr = err = StringIO.StringIO()

        r.schedule_call(0, timer)
        r.add_observer(run_loop_twice, 'after_waiting')
        r.run()

        err.seek(0)
        sys.stderr = saved

        assert len(observed) == 1
        assert err.read()
        assert not r.running

    def test_timer_system_exception(self):
        r = runloop.RunLoop()
        def timer():
            raise SystemExit

        r.schedule_call(0, timer)

        caught = []
        try:
            r.run()
        except SystemExit:
            caught.append(True)

        assert caught
        assert not r.running

if __name__ == '__main__':
    unittest.main()






import syslog
import logging


def file_logger(filename):
    """Create a logger. This sucks, the logging module sucks, but
    it'll do for now.
    """
    handler = logging.FileHandler(filename)
    formatter = logging.Formatter()
    handler.setFormatter(formatter)
    log = logging.getLogger(filename)
    log.addHandler(handler)
    log.setLevel(logging.DEBUG)
    return log, handler


class LineLogger(object):
    towrite = ''
    def __init__(self, emit=None):
        self.emit = emit

    def write(self, stuff):
        self.towrite += stuff
        if '\n' in self.towrite:
            self.flush()

    def flush(self):
        try:
            newline = self.towrite.index('\n')
        except ValueError:
            newline = len(self.towrite)
        while True:
            self.emit(self.towrite[:newline])
            self.towrite = self.towrite[newline+1:]
            try:
                newline = self.towrite.index('\n')
            except ValueError:
                break

    def close(self):
        pass


class SysLogger(LineLogger):
    """A file-like object which writes to syslog. Can be inserted
    as sys.stdin and sys.stderr to have logging output redirected
    to syslog.
    """
    def __init__(self, priority):
        self.priority = priority

    def emit(self, line):
        syslog.syslog(self.priority, line)


class TeeLogger(LineLogger):
    def __init__(self, one, two):
        self.one, self.two = one, two

    def emit(self, line):
        self.one.emit(line)
        self.two.emit(line)


class FileLogger(LineLogger):
    def __init__(self, file):
        self.file = file

    def emit(self, line):
        self.file.write(line)




import sys
import select
import socket
import errno
import traceback
from time import sleep

from eventlet import greenlib
from eventlet.runloop import RunLoop, Timer

import greenlet

EXC_MASK = select.POLLERR | select.POLLHUP | select.POLLNVAL
READ_MASK = select.POLLIN
WRITE_MASK = select.POLLOUT

class Hub(object):
    def __init__(self):
        self.runloop = RunLoop(self.wait)
        self.descriptor_queue = {}
        self.descriptors = {}
        self.greenlet = None
        self.poll = select.poll()

    def stop(self):
        self.process_queue()
        self.runloop.abort()
        if self.greenlet is not greenlet.getcurrent():
            self.switch()

    def schedule_call(self, *args, **kw):
        return self.runloop.schedule_call(*args, **kw)

    def switch(self):
        if not self.greenlet:
            self.greenlet = greenlib.tracked_greenlet()
            args = ((self.runloop.run,),)
        else:
            args = ()
        try:
            greenlet.getcurrent().parent = self.greenlet
        except ValueError:
            pass
        return greenlib.switch(self.greenlet, *args)

    def add_descriptor(self, fileno, read=None, write=None, exc=None):
        if fileno in self.descriptor_queue:
            oread, owrite, oexc = self.descriptor_queue[fileno]
            read, write, exc = read or oread, write or owrite, exc or oexc
        self.descriptor_queue[fileno] = read, write, exc
        
    def remove_descriptor(self, fileno):
        self.descriptor_queue[fileno] = None, None, None

    def exc_descriptor(self, fileno):
        # We must handle two cases here, the descriptor
        # may be changing or removing its exc handler
        # in the queue, or it may be waiting on the queue.
        exc = None
        try:
            exc = self.descriptor_queue[fileno][2]
        except KeyError:
            try:
                exc = self.descriptors[fileno][2]
            except KeyError:
                pass
        if exc is not None:
            try:
                exc(fileno)
            except self.runloop.SYSTEM_EXCEPTIONS:
                self.squelch_exception(fileno, sys.exc_info())

    def squelch_exception(self, fileno, exc_info):
        traceback.print_exception(*exc_info)
        print >>sys.stderr, "Removing descriptor: %r" % (fileno,)
        try:
            self.remove_descriptor(fileno)
        except Exception, e:
            print >>sys.stderr, "Exception while removing descriptor! %r" % (e,)

    def process_queue(self):
        d = self.descriptors
        reg = self.poll.register
        unreg = self.poll.unregister
        rm = READ_MASK
        wm = WRITE_MASK
        for fileno, rwe in self.descriptor_queue.iteritems():
            read, write, exc = rwe
            if read is None and write is None and exc is None:
                try:
                    del d[fileno]
                except KeyError:
                    pass
                else:
                    try:
                        unreg(fileno)
                    except socket.error:
#                        print "squelched socket err on unreg", fileno
                        pass
            else:
                mask = 0
                if read is not None:
                    mask |= rm
                if write is not None:
                    mask |= wm
                oldmask = 0
                try:
                    oldr, oldw, olde = d[fileno]
                except KeyError:
                    pass
                else:
                    if oldr is not None:
                        oldmask |= rm
                    if oldw is not None:
                        oldmask |= wm
                if mask != oldmask:
                    reg(fileno, mask)
                d[fileno] = rwe
        self.descriptor_queue.clear()
        
    def wait(self, seconds=None):
        self.process_queue()

        if not self.descriptors:
            if seconds:
                sleep(seconds)
            return
        try:
            presult = self.poll.poll(seconds * 1000.0)
        except select.error, e:
            if e.args[0] == errno.EINTR:
                return
            raise
        SYSTEM_EXCEPTIONS = self.runloop.SYSTEM_EXCEPTIONS
        dct = self.descriptors

        for fileno, event in presult:
            try:
                read, write, exc = dct[fileno]
            except KeyError:
                continue

            if read is not None and event & READ_MASK:
                try:
                    read(fileno)
                except SYSTEM_EXCEPTIONS:
                    raise
                except:
                    self.squelch_exception(fileno, sys.exc_info())
            elif exc is not None and event & EXC_MASK:
                try:
                    exc(fileno)
                except SYSTEM_EXCEPTIONS:
                    raise
                except:
                    self.squelch_exception(fileno, sys.exc_info())

            if write is not None and event & WRITE_MASK:
                try:
                    write(fileno)
                except SYSTEM_EXCEPTIONS:
                    raise
                except:
                    self.squelch_exception(fileno, sys.exc_info())



"""What does mu mean?

Nothing
"""

import os, types, urllib, cgi, cStringIO, sys, tempfile, Cookie
import mimetypes, time, traceback

from datetime import datetime

HTTP_TIME = '%a, %d %b %Y %H:%M:%S GMT'


def http_time(timestamp):
    return datetime.utcfromtimestamp(timestamp).strftime(HTTP_TIME)


class xml(object):
    """An object which contains only xml bytes.
    """
    def __init__(self, string):
        self.string = string


####################
####################
## Argument Converters
####################
####################


def arguments(*args):
    """A Decorator which you wrap around a function or a method to indicate
    which magical objects you want passed to your functions which are
    embedded in a Resource's template.

    The magical values you can pass are:
        REQUEST
        SITE
        RESOURCE
        METHOD
        URI

    For example, to pass a function the SiteMap instance, do this:

        @arguments(SITE)
        def foo(site):
            print "The site!"

    The values which you pass a string argument to get variables out of the
    request environment (which also take a default) are:

        QUERY
        QUERIES
        ARG
        ARGS
        COOKIE
        HEADER

    For example, to pass a query parameter to a function, with a default
    if the parameter is not present, do this:

        @arguments(QUERY('foo', 'default'))
        def foo(foo):
            print foo

    All the magic objects which take string values have a __getattr__
    implementation for syntatic convenience when you do not need to
    pass a default value (None will be used as the default value):

        @arguments(QUERY.foo)
        def foo(foo):
            print foo
    """
    def decorate(func):
        func.arguments = args
        return func
    return decorate


def rarguments(return_converter, *args):
    def decorate(func):
        func.returnConverter = return_converter
        func.arguments = args
        return func
    return decorate


class REQUEST(object):
    """The Request object representing the current web request.
    """


class SITE(object):
    """The SiteMap object in which the resource which is currently being
    rendered resides.
    """


class RESOURCE(object):
    """The currently-being-rendered resource.
    """


class METHOD(object):
    """The method of the current request.
    """


class URI(object):
    """The URI of the current request.
    """


class Thing(object):
    """A lazy thing. Lazy things have a name and a default
    value, and are looked up lazily from the request environment.
    """
    def __init__(self, name, default=None):
        self.name = name
        self.default = default

    def __getattr__(self, name):
        if name.startswith('_'):
            raise AttributeError, name
        return type(self)(name)

    def __call__(self, name, default=None):
        return type(self)(name, default)


class QUERY(Thing):
    """A query argument (url parameter)
    """


class QUERIES(Thing):
    """Produce a generator of all query arguments with the given name
    """


class ARG(Thing):
    """A form post variable.
    """


class ARGS(Thing):
    """All form post variables with the given name.
    """


class COOKIE(Thing):
    """A cookie with the given name.
    """


class HEADER(Thing):
    """A header with the given name.
    """


def getQueries(request, arg):
    queries = tuple(request.get_query(arg.name))
    if not queries:
        return arg.default
    return queries


def getQuery(request, arg):
    queries = getQueries(request, arg)
    if queries == arg.default:
        return arg.default
    return queries[0]


def getArgs(request, arg):
    val = request.get_arg_list(arg.name)
    if not val:
        return arg.default
    return val


def getCookie(request, arg):
    """get a cookie woot.
    """
    if not hasattr(request, '_simple_cookie'):
        cookie = request.get_header('Cookie')
        if not cookie:
            return arg.default
        c = Cookie.SimpleCookie()
        c.load(cookie)
        request._simple_cookie = c
    cook_value = request._simple_cookie.get(arg.name)
    if cook_value and cook_value.value:
        return cook_value.value
    return arg.default


## Argument converters are passed to the "arguments" decorator.
## Immediately before the decorated function is called, they are invoked.
## They are passed information about the current request
## and they should return the lazy value they represent.
argumentConverters = {
    REQUEST: lambda request, arg: request,
    SITE: lambda request, arg: request.site,
    RESOURCE: lambda request, arg: request.resource,
    METHOD: lambda request, arg: request.method(),
    URI: lambda request, arg: request.uri(),
    QUERY: getQuery,
    QUERIES: getQueries,
    ARG: lambda request, arg: request.get_arg(arg.name, arg.default),
    ARGS: getArgs,
    COOKIE: getCookie,
    HEADER: lambda request, arg: request.get_header(arg.name, arg.default),
}


def registerArgumentConverter(argType, converter):
    argumentConverters[argType] = converter


## These things are all prototype objects; they have __getattr__
## and __call__ factories which are used for generating clones
## of them. The lines below make the names that people use
## into instances instead of classes.
REQUEST = REQUEST()
SITE = SITE()
RESOURCE = RESOURCE()
METHOD = METHOD()
URI = URI()
QUERY = QUERY('')
QUERIES = QUERIES('')
ARG = ARG('')
ARGS = ARGS('')
COOKIE = COOKIE('')
HEADER = HEADER('')


####################
####################
## End Argument Converters
####################
####################


####################
####################
## Rendering Converters
####################
####################


## Rendering converters
## When objects are placed into a DOM which is rendered to be
## sent to the client as HTML, they are run through a series
## of rendering converters until nothing but a single "xml"
## instance is left.


def convertFunction(request, function):
    args = []
    for arg in getattr(function, 'arguments', ()):
        converter = argumentConverters.get(type(arg))
        if converter is None:
            args.append(arg)
        else:
            args.append(converter(request, arg))

    result = function(*args)

    converter = getattr(function, 'returnConverter', None)
    if converter is not None:
        result = converter(result)

    return result


convertSequence = lambda request, sequence: xml(
    ''.join([request.convert(request, x) for x in sequence]))


convertNumber = lambda request, number: xml(str(number))


class show(object):
    """A marker which will lazily look up a show_* method
    from the currently-being-rendered resource when encountered
    in a template.
    """
    def __init__(self, name, args=()):
        self.name = name
        self.args = args

    def __getattr__(self, name):
        if name.startswith('_'):
            raise AttributeError, name
        return type(self)(name)

    def __call__(self, *args):
        return type(self)(self.name, self.args + args)

    def __iter__(self):
        return iter(())

    def clone(self):
        return type(self)(self.name, self.args)

    def __repr__(self):
        return "show(%r, %r)" % (self.name, self.args)


def convertShow(request, theDirective):
    def convert(resource, *args):
        rv = getattr(resource, "show_%s" % (theDirective.name, ))
        rv.im_func.arguments = args
        return rv
    return arguments(RESOURCE, *theDirective.args)(convert)


class slot(object):
    """A marker which will be removed in rendering unless it
    is filled by calling stan.fill(name, value), in which case
    value will be rendered in it's place.
    """
    def __init__(self, name):
        self.slot = name
        self.show = ''

    def __getattr__(self, name):
        if name.startswith('_'):
            raise AttributeError, name
        return type(self)(name)

    def __iter__(self):
        return iter(())

    def clone(self):
        return type(self)(self.slot)

    def __repr__(self):
        return "slot(%r, %r)" % (self.slot, self.show)


def find(someStan, targetPattern):
    """Find a node marked with the given pattern, "targetPattern",
    in a DOM object, "someStan"
    """
    if isinstance(
        someStan, type(pattern)) and someStan.pattern == targetPattern:
        return type(pattern)('', clone_children(someStan.children))
    for child in getattr(someStan, 'children', ()):
        result = find(child, targetPattern)
        if result is not None:
            return result


def rfill(s, **kw):
    if isinstance(s, type(pattern)): return ''
    slot = getattr(s, 'slot', None)
    if slot in kw and not s.show:
        s.show = kw[slot]
    else:
        for child in (getattr(s, 'children', ())):
            rfill(child, **kw)


def fill(s, **kw):
    children = clone_children(s.children)
    for child in children:
        rfill(child, **kw)
    return type(pattern)('', children)


class pattern(object):
    """A marker which lets you find this branch of the tree
    again later.
    """
    def __init__(self, name, children=()):
        self.pattern = name
        if name:
            self.show = ''
        self.children = children

    def __getattr__(self, name):
        if name.startswith('_'):
            raise AttributeError, name
        return type(self)(name)

    def __getitem__(self, args):
        if not isinstance(args, (tuple, list)):
            args = (args, )
        return type(self)(
            self.pattern, [x for x in self.children] + list(args))

    def clone(self):
        return type(self)(
            self.pattern, clone_children(self.children))

    fill = fill
    find = find

    def __repr__(self):
        return "pattern(%r, %r)" % (
            self.pattern, self.children)


def convertPattern(request, pattern):
    ## Pattern originals left in the template are elided
    if pattern.pattern:
        return nothing
    return pattern.children


class attr(object):
    def __init__(self, name, children):
        self.name = name
        self.children = children

    def clone(self):
        if isinstance(self.children, basestring):
            children = self.children
        else:
            children = clone_children(self.children)
        return type(self)(self.name, children)

    def __repr__(self):
        return "attr(%r, %r)" % (
            self.name, self.children)


def clone_children(children):
    new_children = []
    for child in children:
        if hasattr(child, 'clone'):
            new_children.append(child.clone())
        elif isinstance(child, (list, tuple)):
            new_children.append(clone_children(child))
        else:
            new_children.append(child)
    return new_children


class stan(object):
    def __init__(
        self, tag, children, clone=False):
        (self.tag, self.children, self.clone_me) = (
            tag, children, clone)

    def __getitem__(self, args):
        """Add child nodes to this tag.
        """
        if not isinstance(args, (tuple, list)):
            args = (args, )

        if self.clone_me:
            return type(self)(
                self.tag, clone_children(self.children) + list(args), False)

        self.children.extend(args)
        return self

    def __call__(self, **kw):
        """Set attributes of this tag. There are two special names
        which are reserved:

            pattern
                Make it possible to find this node later using the
                findPattern function

            show
                When this node is rendered, instead render the
                value passed as the "show" value.
        """
        if self.clone_me:
            attrs = [
                attr(k, v)
                for (k, v) in kw.items()]

            children = clone_children(self.children)

            return type(self)(
                self.tag, attrs + children, False)

        for (k, v) in kw.items():
            self.children.insert(0, attr(k, v))
        return self

    def clone(self):
        return type(self)(
            self.tag, clone_children(self.children), True)

    def __repr__(self):
        return "stan(%r, %r, clone=%r)" % (
            self.tag, self.children, self.clone_me)

    find = find


class cdata(object):
    def __init__(self, data):
        self.data = data


def convertCdata(request, cdata):
    return xml('<![CDATA[%s]]>' % (cdata.data, ))


def tagFactory(tagName):
    return stan(tagName, [], True)


## TODO: Inline elements shouldn't have any whitespace after them or before
## the closing tag.


def convertStan(request, theStan):
    ## XXX this probably isn't necessary
    request.tag = theStan

    attrs = ''
    numattrs = 0
    for att in theStan.children:
        if not isinstance(att, attr):
            break
        numattrs += 1
        name = att.name
        if name.startswith('_'):
            name = name[1:]
        attrs += ' %s="%s"' % (
            name, request.convert(
                request, att.children).replace('"', '&quot;'))

    depth = getattr(request, 'depth', 0)
    indent = '  ' * depth
    request.depth = depth + 1
    if (theStan.tag in singleton_elements and not [
        x for x in theStan.children[numattrs:] if x]
    ):
        template = """<%(tag)s%(attrs)s />"""
    elif theStan.tag in inline_elements:
        template = """<%(tag)s%(attrs)s>%(children)s</%(tag)s>"""
    else:
        template = """
%(indent)s<%(tag)s%(attrs)s>
%(indent)s  %(children)s
%(indent)s</%(tag)s>
"""
    result = template % dict(
        indent=indent, tag=theStan.tag, attrs=attrs,
        children=request.convert(
            request, theStan.children[numattrs:]).strip())

    request.depth -= 1
    return xml(result)


inline_elements = [
    'a', 'abbr', 'acronym', 'b', 'basefont', 'bdo', 'big', 'br', 'cite',
    'code', 'dfn', 'em', 'font', 'i', 'img', 'input', 'kbd', 'label',
    'q', 's', 'samp', 'select', 'small', 'span', 'strike', 'strong',
    'sub', 'sup', 'textarea', 'tt', 'u', 'var', 'pre']
inline_elements = dict([(x, True) for x in inline_elements])


singleton_elements = {}


tags_to_create = [
'a','abbr','acronym','address','applet','area','b','base','basefont','bdo',
'big','blockquote','body','br','button','caption','center','cite','code',
'col','colgroup','dd','dfn','div','dl','dt','em','fieldset','font','form',
'frame','frameset','h1','h2','h3','h4','h5','h6','head','hr','html','i',
'iframe','img','input','ins','isindex','kbd','label','legend','li','link',
'menu','meta','noframes','noscript','ol','optgroup','option','p','param',
'pre','q','s','samp','script','select','small','span','strike','strong',
'style','sub','sup','table','tbody','td','textarea','tfoot','th','thead',
'title','tr','tt','u','ul','var'
]


class tags(object):
    """A namespace for tags, so one can say "from ultramini import tags"
    and have access to all tags as tags.html, tags.foo, etc
    """
    pass


for tag in tags_to_create:
    T = tagFactory(tag)
    globals()[tag] = T
    setattr(tags, tag, T)
del tag
del T
del tags_to_create


nothing = xml('')


converters = {
    type(None): lambda request, n: xml('None'),
    unicode: lambda request, uni: uni.encode('utf8'),
    list: convertSequence,
    tuple: convertSequence,
    types.GeneratorType: convertSequence,
    types.FunctionType: convertFunction,
    types.MethodType: convertFunction,
    types.UnboundMethodType: convertFunction,
    int: convertNumber,
    float: convertNumber,
    long: convertNumber,
    show: convertShow,
    stan: convertStan,
    slot: lambda request, slot: slot.show,
    pattern: convertPattern,
    cdata: convertCdata,
    str: lambda request, string: xml(
        string.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;"))
}
converters.update(argumentConverters)


show = show('')
slot = slot('')
pattern = pattern('')

def registerConverter(convertType, converter):
    """Call this to register a converter for one of your custom types.
    """
    converters[convertType] = converter


def convert(request, stuff):
    """Recursively apply stuff converters until we get an xml instance.
    """
    while not isinstance(stuff, xml):
        for cls in getattr(stuff, '__mro__', (type(stuff), )):
            convert = converters.get(cls)
            if convert is not None:
                stuff = convert(request, stuff)
                break
        else:
            raise RuntimeError, "Converter for type %r (%r) not found." % (
                type(stuff), stuff)

    return stuff.string


####################
####################
## End Rendering Converters
####################
####################



####################
####################
## Incoming Type Converters
####################
####################


## When arguments come in (as GET url parameters or POST arguments)
## they can be converted from strings to Python types using incoming
## type converters. To use them, wrap a get_* or post_* method
## using the annotate decorator. When a get_ or post_ method
## is called (by having a corresponding ?get= parameter or action
## argument) the arguments named in the decorator will be converted
## before they are passed to the method.


def rannotate(returnConverter, **kw):
    """annotate with a return value converter
    """
    def decorate(method):
        method.returnConverter = returnConverter
        method.types = kw
        return method
    return decorate


def annotate(**kw):
    """A decorator which annotates the types of arguments an action
    method will receive. Used by 'get' and 'post' methods; for
    example, to expose a method to be called by placing ?get=foo
    in the query string with a single parameter bar of type int:

    @annotate(bar=int)
    def get_foo(self, bar):
        print bar

    To expose a method to a POST with an action argument and a
    bar argument of type int:

    @annotate(bar=int)
    def post_foo(self, bar):
        print bar

    To express lists of things:
    @annotate(bar=[int])
    def post_foo(self, bar):
        for someInt in bar:
            print someInt

    TODO: Dicts of things should be expressed by presenting
    arguments with a common prefix separated by a dot. For
    example:

    ?get=foo&someDict.bar=1&someDict.baz=foo

    Would call:

    @annotate(someDict={bar: int, baz: str})
    def get_foo(self, someDict):
        print someDict['bar']
        print someDict['baz']

    This is still TODO. (Perhaps colon instead of dot?)
    """
    def decorate(method):
        method.returnConverter = None
        method.types = kw
        return method
    return decorate


def convertList(request, theList):
    request.target = request.target[0]
    if not isinstance(theList, list):
        theList = [theList]
    return [convertTypes(request, item) for item in theList]


def convertTuple(request, theTuple):
    stuff = []
    for i, subtype in enumerate(request.target):
        request.target = subtype
        stuff.append(convertTypes(request, theTuple[i]))
    return tuple(stuff)


def convertDict(request, nothing):
    theDict = {}
    for field in request.get_field_storage().list:
        if field.name.startswith(request.target_name + '.'):
            theDict[field.name.split('.', 1)[-1]] = field.value
    target = request.target
    result = {}
    for (key, value) in target.items():
        request.target = value
        result[key] = convertTypes(request, theDict[key])
    return result


typeConverters = converters.copy()
typeConverters.update({
    int: lambda target, value: int(value),
    float: lambda target, value: float(value),
    list: convertList,
    tuple: convertTuple,
    dict: convertDict,
    str: lambda target, value: value,
    unicode: lambda target, value: value.decode('utf8'),
    types.FunctionType: lambda request, value: request.target(request, value),
})


def registerTypeConverter(convertType, converter):
    typeConverters[convertType] = converter


def convertTypes(request, value):
    target = request.target
    converter = typeConverters.get(type(target))
    if converter is None:
        converter = typeConverters.get(target)
    if converter is None:
        raise ValueError, "No converter for type %s (%s)" % (target, value)
    return converter(request, value)


def applyWithTypeConversion(callable_method, request):
    types = getattr(callable_method, 'types', {})
    try:
        converted = {}
        fs = request.get_field_storage()
        for (key, value) in types.items():
            request.target_name = key
            request.target = value
            if fs.has_key(key):
                val = fs[key]
                if isinstance(val, list):
                    val = [v.value for v in val]
                else:
                    val = val.value
                converted[key] = convertTypes(request, val)
            else:
                try:
                    converted[key] = convertTypes(
                        request, getattr(value, 'default', None))
                except (TypeError, ValueError):
                    converted[key] = getattr(value, 'default', None)

        result = callable_method(**converted)
    except TypeError:
        print "Error calling action method", callable_method
        raise
    if result is None:
        result = ''
    returnConverter = getattr(callable_method, 'returnConverter', None)
    if returnConverter:
        result = returnConverter(result)
    return result


####################
####################
## End Incoming Converters
####################
####################


####################
####################
## Resource Converers
####################
####################

resourceConverters = {
    str: lambda s: File(s)
}


def registerResourceConverter(t, c):
    resourceConverters[t] = c


####################
####################
## End Resource Converers
####################
####################


####################
####################
## URL Traversal
####################
####################


## SiteMap has a root Resource and traverses the URL using the Resource
## interface.


class SiteMap(object):
    """An object which is capable of matching Resource objects to URLs and
    delegating rendering responsibility to the matched object.

    This class implements the corohttpd handler interface: match
    and handle_request. It adapts this interface to something more similar
    to the Nevow URL traversal interface: findChild and handle (like
    locateChild and renderHTTP in Nevow).
    """
    def __init__(self, root, cache=False):
        self.resources = {}
        self.root = root
        root.child_ = root
        self.cache = cache

    def handle_request(self, request): 
        request.convert = convert
        try:
            self._really_handle_request(request)
        except:
            request.protocol.server.log_exception(*sys.exc_info())
            ## Relative import
            import htmlexception
            request.response(500)
            request.write(tags.html[tags.body[htmlexception.formatException()]])
        del request.convert

    def _really_handle_request(self, request):
        request.site = self
        remaining = tuple([urllib.unquote_plus(x) for x in request._path.split('/')[1:]])
        ## Cache fastpath
        if remaining in self.resources:
            request.resource = self.resources[remaining]
            request.segments = remaining
            return True

        child = self.root

        while child is not None and remaining:
            child, handled, remaining = child.findChild(request, remaining)
            if type(child) in resourceConverters:
                child = resourceConverters[type(child)](child)

        if child is None:
            request.response(404)
            request.write('Not Found')
        else:
            ## Allow resources to delegate to someone else,
            ## even if they have been selected as the target of rendering
            newChild = child.willHandle(request)
            if newChild is not child:
                child = newChild
            request.segments = handled
            request.resource = child
            resource = request.resource
            resource.handle(request)
            if self.cache and resource.cache:
                self.resources[request.segments] = request.resource
            request.resource = None


class Resource(object):
    cache = True
    debug = False
    contentType = 'text/html'
    template = "Resources must provide a template attribute."

    def __call__(self, request, name):
        return self

    def willHandle(self, request):
        """This Resource is about to handle a request.
        If it wants to delegate to another Resource, it can return it here.
        """
        return self

    def findChild(self, request, segments):
        """External URL segment traversal API. This method MUST always
        return a tuple of:

            (child, handled, remaining)

        child may be None to indicate a 404. Handled should be a tuple
        of URL segments which were handled by this call to findChild;
        remaining should be a tuple of URL segments which were not
        handled by this call to findChild.

        findChild can be overriden to implement fancy URL traversal
        mechanisms such as handling multiple segments in one call,
        doing an internal server-side redirect to another resource and
        passing it segments to handle, or delegating this segment
        to another resource entirely. However, for most common use
        cases, you will not override findChild.

        Any methods or attributes named child_* will be mapped to
        URL segments. For example, if an instance of Root is set
        as the root object, the urls "/foo" and "/bar" will be valid:

        class Root(Resource):
            child_foo = Resource()

            def child_bar(self, request, bar):
                return Resource()

        Finally, if a childFactory method is defined it will be called
        with a single URL segment:

        class Root(Resource):
            def childFactory(self, request, childName):
                ## For URL "/foo" childName will be "foo"
                return Resource()
        """
        ## Refresh our code
        selfType = type(self)
        if self.debug and selfType.__module__ != '__main__':
            self.__class__ = getattr(
                reload(sys.modules[selfType.__module__]), selfType.__name__)

        current, remaining = segments[0], segments[1:]
        childFactory = getattr(self, 'child_%s' % (current, ), self.childFactory)
        if type(childFactory) in resourceConverters:
            childFactory = resourceConverters[type(childFactory)](childFactory)
        return childFactory(request, current), (current, ), remaining

    def childFactory(self, request, childName):
        """Override this to produce instances of Resource to represent the next
        url segment below self. The next segment is provided in childName.
        """
        return None

    def handle(self, request):
        ## Refresh our code
        selfType = type(self)
        if self.debug and selfType.__module__ != '__main__':
            self.__class__ = getattr(
                reload(sys.modules[selfType.__module__]), selfType.__name__)

        ## Handle the request
        request['Content-Type'] = self.contentType
        handler = getattr(
            self, "handle_%s" % (request.method(), ), self.handle_default)
        handler(request)

    def handle_default(self, request):
        request.push(
            "<html><body>No handler for method %r present.</body></html>" %
            (request.method(), ))

    def handle_get(self, request):
        get_method = self.getAction('get', request.get_field_storage())
        if get_method is None:
            request.write(self.template)
        else:
            result = applyWithTypeConversion(get_method, request)
            request.write(result)

    def getFieldStorage(self, request, data, headers, environ):
        """Given a file-like object, data, a dict of headers, and an environ dict,
        return an instance of cgi.FieldStorage or something which acts quite like
        one. This is provided as a hook for subclasses to override if they want
        to parse the incoming POST data differently than a basic FieldStorage
        does. For example, imgsrv/server overrides this to trick FieldStorage
        into saving binary parts directly into the image file directory, to avoid
        having to copy the file from a temporary file into the final location.
        """
        return cgi.FieldStorage(data, headers, environ=environ)

    def getAction(self, key, fs):
        if isinstance(fs, dict):
            value = dict.get(key, 'default')
        else:
            try:
                value = fs.getfirst(key, 'default')
            except TypeError:
                value = 'default'
        return getattr(self, '%s_%s' % (key, value), None)

    def handle_post(self, request):
        fs = request.get_field_storage()
        post_method = self.getAction('post', fs)
        if post_method is None:
            url = self.post(request, fs)
        else:
            url = applyWithTypeConversion(post_method, request)

        if url is not None:
            request['Location'] = request.convert(request, url)
            request.response(303)
            request.write('')
            return
        request.write(self.template)

    def post(self, request, form):
        """post

        Override this to handle a form post.
        Return a URL to be redirected to.

        request: An HttpRequest instance representing the place the form
                  was posted to.
        form: A cgi.FieldStorage instance representing the posted form.
        """
        return request.uri()


class MovedPermanently(Resource):
    cache = False
    def __init__(self, location):
        self.location = location

    def get_default(self, req):
        req.set_header(
            'Location',
            self.location)
        req.response(301) # Moved Permanently
        return self.location
    get_default = annotate(req=REQUEST)(get_default)


class Directory(Resource):
    def __init__(self, path):
        self.path = path

    def findChild(self, req, segments):
        full_path = os.path.join(*(self.path, ) + segments)
        if os.path.exists(full_path):
            return (
                File(full_path, mimetypes.guess_type(full_path)),
                segments, ())
        return (None, (), segments)


class File(Resource):
    def __init__(self, path, content_type='text/html'):
        self.path = path
        self.content_type = content_type

    def get_default(self, req):
        f = file(self.path, 'rb')
        finfo = os.stat(self.path)
        req.set_header('Content-Type', self.content_type)
        modtime = http_time(finfo.st_mtime)
        req.set_header('Last-Modified', modtime)

#        if req.get_header('If-Modified-Since') == modtime:
#            req.response(304)
#            req.push('')

        req.set_header('Content-Length', str(finfo.st_size))
        return xml(f.read())

        block = f.read(32*1024)
        if not block:
            req.send_error(204)
        else:
            while True:
                req.write_chunked(xml(block))
                block = f.read(32*1024)
                if not block:
                    break
    get_default = annotate(req=REQUEST)(get_default)


class Data(Resource):
    def __init__(self, data, content_type='text/html'):
        self.data = data
        self.content_type = content_type

    def get_default(self, req):
        req.set_header('Content-Type', self.content_type)
        req.set_header('Content-Length', len(self.data))
        req.write(self.data)
    get_default = annotate(req=REQUEST)(get_default)




import sys
import select
import kqueue
import traceback
from errno import EBADF

from eventlet import greenlib
from eventlet.runloop import RunLoop, Timer

import greenlet

class Hub(object):
    def __init__(self):
        self.runloop = RunLoop(self.wait)
        self.descriptor_queue = {}
        self.descriptors = {}
        self.greenlet = None
        self.kfd = None

    def stop(self):
        self.process_queue()
        self.descriptors, self.descriptor_queue = self.descriptor_queue, {}
        os.close(self.kfd)
        self.kfd = None
        self.runloop.abort()
        if self.greenlet is not greenlet.getcurrent():
            self.switch()

    def schedule_call(self, *args, **kw):
        return self.runloop.schedule_call(*args, **kw)

    def switch(self):
        if not self.greenlet:
            self.greenlet = greenlib.tracked_greenlet()
            args = ((self.runloop.run,),)
        else:
            args = ()
        try:
            greenlet.getcurrent().parent = self.greenlet
        except ValueError:
            pass
        return greenlib.switch(self.greenlet, *args)

    def add_descriptor(self, fileno, read=None, write=None, exc=None):
        self.descriptor_queue[fileno] = read, write, exc
        
    def remove_descriptor(self, fileno):
        self.descriptor_queue[fileno] = None, None, None

    def exc_descriptor(self, fileno):
        # We must handle two cases here, the descriptor
        # may be changing or removing its exc handler
        # in the queue, or it may be waiting on the queue.
        exc = None
        try:
            exc = self.descriptor_queue[fileno][2]
        except KeyError:
            try:
                exc = self.descriptors[fileno][2]
            except KeyError:
                pass
        if exc is not None:
            try:
                exc()
            except self.runloop.SYSTEM_EXCEPTIONS:
                self.squelch_exception(fileno, sys.exc_info())

    def squelch_exception(self, fileno, exc_info):
        traceback.print_exception(*exc_info)
        print >>sys.stderr, "Removing descriptor: %r" % (fileno,)
        try:
            self.remove_descriptor(fileno)
        except Exception, e:
            print >>sys.stderr, "Exception while removing descriptor! %r" % (e,)

    def process_queue(self):
        if self.kfd is None:
            self.kfd = kqueue.kqueue()
        d = self.descriptors

        E_R = kqueue.EVFILT_READ
        E_W = kqueue.EVFILT_WRITE
        E = kqueue.Event
        E_ADD = kqueue.EV_ADD
        E_DEL = kqueue.EV_DELETE
        
        kevent = kqueue.kevent
        kfd = self.kfd
        for fileno, rwe in self.descriptor_queue.iteritems():
            read, write, exc = rwe
            if read is None and write is None and exc is None:
                try:
                    read, write, exc = d.pop(fileno)
                except KeyError:
                    pass
                else:
                    l = []
                    if read is not None:
                        l.append(E(fileno, E_R, E_DEL))
                    if write is not None:
                        l.append(E(fileno, E_W, E_DEL))
                    if l:
                        try:
                            kevent(kfd, l, 0, 0)
                        except OSError, e:
                            if e[0] != EBADF:
                                raise
            else:
                l = []
                try:
                    oldr, oldw, olde = d[fileno]
                except KeyError:
                    pass
                else:
                    if oldr is not None:
                        if read is None:
                            l.append(E(fileno, E_R, E_DEL))
                        else:
                            read = None
                    if oldw is not None:
                        if write is None:
                            l.append(E(fileno, E_W, E_DEL))
                        else:
                            write = None
                if read is not None:
                    l.append(E(fileno, E_R, E_ADD))
                if write is not None:
                    l.append(E(fileno, E_W, E_ADD))
                if l:
                    try:
                        kevent(kfd, l, 0, 0)
                    except OSError, e:
                        if e[0] != EBADF:
                            raise
                        try:
                            del d[fileno]
                        except KeyError:
                            pass
                        if exc is not None:
                            try:
                                exc(fileno)
                            except SYSTEM_EXCEPTIONS:
                                raise
                            except:
                                self.squelch_exception(fileno, sys.exc_info())
                        continue
                d[fileno] = rwe
        self.descriptor_queue.clear()
        
    def wait(self, seconds=None):

        self.process_queue()

        if seconds is not None:
            seconds *= 1000000000.0
        dct = self.descriptors
        events = kqueue.kevent(self.kfd, [], len(dct), seconds)

        SYSTEM_EXCEPTIONS = self.runloop.SYSTEM_EXCEPTIONS

        E_R = kqueue.EVFILT_READ
        E_W = kqueue.EVFILT_WRITE
        E_EOF = kqueue.EV_EOF

        for e in events:
            fileno = e.ident
            event = e.filter

            try:
                read, write, exc = dct[fileno]
            except KeyError:
                continue

            if read is not None and event == E_R:
                try:
                    read(fileno)
                except SYSTEM_EXCEPTIONS:
                    raise
                except:
                    self.squelch_exception(fileno, sys.exc_info())
            elif exc is not None and e.fflags & E_EOF:
                try:
                    exc(fileno)
                except SYSTEM_EXCEPTIONS:
                    raise
                except:
                    self.squelch_exception(fileno, sys.exc_info())

            if write is not None and event == E_W:
                try:
                    write(fileno)
                except SYSTEM_EXCEPTIONS:
                    raise
                except:
                    self.squelch_exception(fileno, sys.exc_info())
